

#' Calculate the length of small stream rearing habitats
#'
#' Use pool percent to break into pool/riffle habitats for summer and winter
#' rearing. Add beaver ponds, reducing stream length by beaver pond length
#'
#' @param df_clean_ss Output of hab_clean_ss()
#' @param beaver_model Code for beaver model to be run: "BIP" or "SP"
#'
#' @export

hab_calc_ss_length <- function(df_clean_ss, beaver_model = c("BIP", "SP")) {

  if(beaver_model == "BIP"){
    ss_beav_potent <- calc_BIP(df_clean_ss, slope, BF_width, fpw)
  } else if (beaver_model == "SP"){
    ss_beav_potent <- calc_pollock(df_clean_ss, slope, Area_km2)
  } else stop("Beaver model must be BIP or SP.")

  ss_beav_potent %>%
    left_join.(scenarios_df, by = c("noaaid", "scenario")) %>%
    left_join.(pool_ratios, by = c("lc", "slope_class")) %>%
    mutate.(
      pool_perc = ifelse(
        is.na(pool_perc) & wood_intensity_scalar > 0,
        0,
        pool_perc
      ),
      pool_perc_scenario = apply_rest_action(
        pool_perc,
        pool_perc_ref,
        rest_perc,
        wood_intensity_scalar
      ),
      beaver_mult = tidytable::if_else.(
        beav_hab, # only apply beaver action to potential habitat
        apply_rest_action(
          curr_beaver_mult,
          hist_beaver_mult,
          rest_perc,
          beaver_intensity_scalar
        ),
        1), # no length removed from unsuitable streams
      # habitat length scalar calculations ----
      # this divides reach into pool, riffle, and beaver pond
      Pool_summer = pool_perc_scenario,
      Riffle_summer = 1 - Pool_summer,
      Pool_winter = Pool_summer * winter_pool_scalar_warm,
      Riffle_winter = 1 - Pool_winter,
      # add beaver ponds, note these are area scalars ----
      Beaver.Pond_summer = apply_rest_action(
        curr_pond_area_per_m,
        hist_pond_area_per_m,
        rest_perc,
        beaver_intensity_scalar) * beav_hab, # beav_hab is a logical. This
      # only occurs in potential beaver habitats
      Beaver.Pond_winter = Beaver.Pond_summer
    ) %>%
    select.(-Habitat) %>%
    tidytable::pivot_longer.(#split habitat and life stage
      Pool_summer:Beaver.Pond_winter,
      names_to = c("Habitat", "life.stage"),
      names_sep = "_",
      values_to = "length_scalar"
    ) %>%
    mutate.(
      length = tidytable::if_else.(
        Habitat == "Beaver.Pond",
        length * length_scalar,
        length * length_scalar * beaver_mult # reduce length for pond
      )) %>%
    select.(
      noaaid,
      subbasin_num,
      Habitat,
      length,
      BF_width,
      life.stage,
      lc,
      slope_class,
      scenario,
      era,
      pop,
      names(scenarios_df)
    )
}
