#' Calculate wood scalar used in capacity and productivity calculations

#' Function used to calculate wood multiplier
#'
#' @param length_bank length of bank habitat within a reach
#' @param length_bar length of bar habitat within a reach
#' @param scalar_bank scalar applied to bank habitat
#' @param scalar_bar scalar applied to bar habitat
#' @export

calc_woodmult <- function(length_bank, length_bar, scalar_bank, scalar_bar) {
  ((length_bank * scalar_bank) + (length_bar * 2 * scalar_bar)) /
    (length_bank + (length_bar * 2))
}


#' Calculate the impact of wood on large rivers summarized at the subbasin level
#'
#' Wood changes capacity, which we reflect here by changing area
#' Wood impact on large rivers differs by life stage
#'
#' @param run_pop Species of interest for this model run
#' @param df_large_river Output of hab_prep_lr()
#' @export

hab_apply_lr_woodmult <- function(run_pop, df_large_river){

  # Make lr_wd_XXXX params available within the function
  list2env(get(paste0('params_hab_', run_pop)), envir = environment())

  # Calculate wood data
  df_large_river %>%
    mutate.(
      woodhab = tidytable::case_when.(
        Habitat == "Bank" ~ "Bank",
        stringr::str_detect(Habitat, 'Bar') ~ "Bar", # lump bars
        TRUE ~ "none")) %>%
    filter.(!woodhab == "none") %>% # exclude HM_Bank (armored)
    tidytable::summarize.(
      length = sum(Length_m),
      .by = c(subbasin_num, woodhab)) %>% # length in each hab and subbasin
    tidytable::pivot_wider.(names_from = woodhab, values_from = length) %>%
    # include all subbasins when joining back in
    tidytable::full_join.(subbasin_names, by = "subbasin_num") %>%
    mutate.(
      woodmult_s = calc_woodmult(Bank, Bar, lr_wd_s_bank, lr_wd_s_bar),
      woodmult_w = calc_woodmult(Bank, Bar, lr_wd_w_bank, lr_wd_w_bar)) %>%
    select.(subbasin_num, woodmult_s, woodmult_w) %>%
    mutate.(
      woodmult_s = impute_mean(woodmult_s),
      woodmult_w = impute_mean(woodmult_w)) %>%
    tidytable::arrange.(subbasin_num) # this fixes a join issue later

}
