#' Calculate capacity
#'
#' @param run_pop which pop to run
#' @param df_lr_area output of hab_calc_lr_area()
#' @param df_ss_area output of hab_calc_ss_area()
#' @param df_fp output of hab_calc_fp_area()
#' @param df_w output of hab_apply_lr_woodmult()
#' @param df_temperature dataframe with temps per reach
#'
#' @export


hab_calc_capacity <- function(run_pop,
                              df_lr_area,
                              df_ss_area,
                              df_fp,
                              df_w,
                              df_temperature){


  # get species-specific density params
  df_density <- get(paste0('params_hab_',run_pop))$density

  # vector of habitats with wood multipliers
  hab_vec <- c(LgRiver_habs, SmStream_habs, "SC_pool", "SC_riffle")

  # data frame of lifestages
  # combine habitat areas
  cap_df <-
    bind_rows.(df_lr_area, df_ss_area, df_fp) %>%
    select.( # strip extra fields
      noaaid,
      Habitat,
      life.stage,
      era,
      scenario,
      subbasin_num,
      Area,
      names(scenarios_df)
    ) %>%
    left_join.(
      df_temperature,
      by = c("noaaid", "scenario", "era")) %>%
    left_join.(barriers_df, by = c("noaaid", "scenario")) %>%
    left_join.(df_w, by = "subbasin_num") %>%
    # split seasons into true life stages via a join table
    left_join.(
      filter.(ls_capacity, pop == run_pop), by = "life.stage") %>%
    mutate.(
      life.stage = life.stage2, # replace field
      pop = run_pop,
      temp = tidytable::if_else.(  #is this a 1st summer chinook population?
        grepl("chinook", pop) & life.stage %in% c("summer", "summer.t"),
        temp_Jn121, # use June temps for 1st summer Chinook
        temp_7DADM # use August temps for other species & yearling chinook
      ),
      tempmult = tidytable::case_when.(
        life.stage %in% c("winter") ~ 1, # mult = 1 during winter
        # mult = 1 for first summer chinook
        grepl("chinook", pop) & life.stage %in% c("summer", "summer.t") ~ 1,
        # mult = 1 for ponds, lakes, and marshes outside of mainstem?
        Habitat %in% c("Lake","Marsh","FP_Pond_lg","FP_Pond_sm") &
          !subbasin_num %in% mainstem_subs ~ 1,
        TRUE ~ hab_temp_func(temp, run_pop) #otherwise, use function
      ),
      woodmult = tidytable::case_when.( #use summer or winter wood multipliers
        confined ~ 1,
        grepl("summer", life.stage) & Habitat %in%  hab_vec~
          apply_rest_action(1, woodmult_s, rest_perc, wood_intensity_scalar),
        grepl("winter", life.stage) & Habitat %in% hab_vec ~
          apply_rest_action(1, woodmult_w, rest_perc, wood_intensity_scalar),
        TRUE ~ 1
      ),
      # bin capacity into 0 (existing zero), or 1 (not zero)
      pass_tot_cap = ceiling(pass_tot_scenario)
    ) %>%
    select.(-pass_tot_scenario) %>%
    left_join.(df_density, by = c("Habitat", "life.stage")) %>%
    mutate.(
      capacity = Area * density * tempmult * woodmult * pass_tot_cap)

  # reset temp mult for chinook
  if (grepl("chinook", run_pop)) {
    cap_df <- cap_df %>%
      mutate.( # use multiplier for summer.t and summer.2
        tempmult = tidytable::case_when.(
          life.stage %in% c('summer.t', 'summer.2') ~
            hab_temp_func(temp, run_pop),
          TRUE ~ 1
        ))
  }
  return(cap_df)
}
