# From 4-inputs.R

#' Calculate the temperature difference from change
#'
#' Chinook use a mdm temp while other species use a 7DADM.
#' This function returns the correct temperature difference from climate change
#' depending on the species.
#'
#' @param pop Define which population to run
#' @export


hab_calc_cc_temp <- function(pop){

  #change in future temperatures due to climate change.  See temp conversion doc for more details
  if (pop %in% c('spring_chinook', 'fall_chinook')) {
    cc_mid_rear <- .9585
    cc_late_rear <- 1.917
  } else {
    cc_mid_rear <- 1.024
    cc_late_rear <- 2.049
  }

  return(lst(cc_mid_rear, cc_late_rear))
}

# From 5-functions

#' Function to calculate prespawn survival based on temperature
#'
#' From the Bowerman paper (2017)
#'
#' @param temp Input temperature
#' @param phos Percent hatchery origin spawner
#' @param b0 Fitted param (given in paper)
#' @param b1 Fitted param (given in paper)
#' @param b2 Fitted param (given in paper)
#' @export

hab_bowerman_ps <- function(temp,
                            phos = 0,
                            b0 = -9.053,
                            b1 = .387,
                            b2 = .029) {
  log_p <- b0 + b1 * temp + b2 * phos
  p <- 1 - plogis(log_p)
  return(p)
}



#' Convert temperature to a scalar
#'
#' Convert a temperature value into a species specific quality scalar.
#'
#' @param temp Temperature
#' @param pop Salmon population
#' @export

hab_temp_func <- function(temp, pop){

  dplyr::case_when(
    # CHINOOK
    grepl("chinook", pop) ~  dplyr::case_when(
      temp < 18 ~ 1, # no impact on temperatures less than 18 C
      # linear for temps 18-24
      temp >= 18 & temp < 24 ~ -(1/6) * temp + 4,
      temp >= 24 ~ 0), # no survival above 24C
    # STEELHEAD
    grepl("steelhead", pop) ~ dplyr::case_when(
      temp <= 17 ~ 1, # no impact on temperatures less than 17 C
      temp > 17 ~ (97.8846 / (1 + exp(-((temp - 24.3522) / -.5033)))) / 100 ),
    # COHO
    pop == 'coho' ~ dplyr::case_when(
      temp < 17 ~ 1, # no impact on temperatures less than 17 C
      # linear for temps 17-28
      temp >= 17 & temp < 28 ~ -(0.09 * temp) + 2.53,
      temp >= 28 ~ 0) # no survival above 28C
  )
}

#' Convert MWMT to MDM
#'
#' Convert temperature in mean weekly maximum to mean daily maximum
#'
#' @param mwmt Tempertaure in mean weekly maximum
#' @export

hab_mwmt_to_mdm <- function(mwmt) {
  .97550 * mwmt
}
