#' Run the Stillaguamish habitat model
#'
#' Run the habitat model based on specified input.
#'
#' @param run_pop which pop to run
#' @param run_scenario which scenario to run.
#' @param run_era which era to run. The choices are hist, curr, mid, late.
#' @param df_temperature default is `harp::temperature_df`. Vary temps here
#' @param df_width default is `harp::width_df`. Vary widths here
#' @param df_fl all flowline reaches for the pop of interest
#' @param pass_mort should partial passage affect prespawn mortality?
#'
#'
#' @section Detailed Options:
#' * `run_pop` can take any of the following values:
#'
#'   * `r paste(sprintf('`````"%s"`````', harp::pops_all), collapse = ", ")`
#'
#' * `run_scenario` can take any of the following values:
#'
#'   * `r paste(sprintf('`````"%s"`````', harp::scenarios_all), collapse = ", ")`
#'
#' * `run_era` can take any of the following values:
#'
#'   * `r paste(sprintf('`````"%s"`````', harp::eras_all), collapse = ", ")`
#'
#' * `beaver_model` can take either of the following values:
#'
#'   * `r paste(sprintf('`````"%s"`````', c("BP", "SP")), collapse = ", ")`
#'
#' @examples
#' # hab_run() requires a run_pop, run_scenario, and run_era.
#' hab_run("winter_steelhead", "Shade", "curr")
#'
#' # other parameters may also be specified
#' hab_run("winter_steelhead", "Shade", "curr", pass_mort = FALSE)
#'
#' # the default temperature, width, and flowline data frames can be replaced
#' hab_run("winter_steelhead", "Shade", "curr", df_width = custom_width_df)
#'
#' @export
#' @md

hab_run <- function(
  run_pop,
  run_scenario,
  run_era,
  beaver_model = "BIP",
  df_temperature = harp::temperature_df,
  df_width = harp::width_df,
  df_fl = harp::flowline_clean[harp::flowline_clean$pop == run_pop, ],
  pass_mort = TRUE){

  #run hab_run_internal
  results <- hab_run_internal(
    run_pop = run_pop,
    run_scenario = run_scenario,
    run_era = run_era,
    beaver_model = beaver_model,
    df_temperature = df_temperature,
    df_width = df_width,
    df_fl = df_fl,
    pass_mort = pass_mort
  )
  message(paste("Successful run of habitat model for",
                run_pop, run_scenario, run_era))
  results

}

#' Run the habitat model (diagnostic)
#'
#' Create habitat scenarios, output data frame of choice.
#' See documentation for \code{\link{hab_run()}} to see general options
#'
#' @param run_pop which pop to run
#' @param run_scenario which scenario to run.
#' @param run_era which era to run. The choices are hist, curr, mid, late.
#' @param df_temperature default is `harp::temperature_df`. Vary temps here
#' @param df_width default is `harp::width_df`. Vary widths here
#' @param df_fl all flowline reaches for the pop of interest
#' @param pass_mort should partial passage affect prespawn mortality?
#' @param return_df which data frame to return
#'
#' @section Output Options for return_df:
#' \itemize{
#'   \item \code{"results"} Full model output, the default
#'   \item \code{"df_ss_clean"} Small streams
#'   \item \code{"df_ss"} Small stream lengths, accounting for beavers
#'   \item \code{"df_ss_area"} Small stream rearing areas
#'   \item \code{"df_sch"} Side channel habitats
#'   \item \code{"df_fp_clean"} Combined floodplain habitats
#'   \item \code{"df_fp"} Floodplain rearing areas
#'   \item \code{"df_lr"} Large rivers
#'   \item \code{"df_bw"} Backwaters
#'   \item \code{"df_lr_clean"} Length-adjusted large rivers
#'   \item \code{"df_w"} Wood multipliers
#'   \item \code{"df_lr_area"} Large river rearing area
#'   \item \code{"df_cap"} Rearing capacity, by reach and habitat
#'   \item \code{"df_prod"} Rearing productivities
#'   \item \code{"df_mvmt"} Movement values, based on scenario
#'   \item \code{"df_spss"} Small stream spawning habitats
#'   \item \code{"df_spfp"} Floodplain spawning habitats
#'   \item \code{"df_splr_clean"} Large river spawning habitats
#'   \item \code{"df_cap_spawn"} Habitat and reach -level spawning capacities
#'   \item \code{"df_cap_egg"} Reach-level spawning capacities
#'   \item \code{"df_prod_e2f"} Subbasin-level incubation productivities
#'   \item \code{"df_prod_prespawn"} Subbasin-level prespawn survivial
#' }
#'
#' @examples
#' # By default, hab_run_diagnostic() runs in the Current scenario and curr era
#' hab_run_diagnostic("coho")
#'
#' # You can specify a different diagnostic scenario as well
#' hab_run_diagnostic("coho", "LR")
#' hab_run_diagnostic("coho", run_scenario = "LR")
#'
#' # Set return_df to return any data frame used in the habitat model.
#' hab_run_diagnostic("fall_chinook", return_df = "df_cap_spawn")
#'
#' # If a return data frame is not specified, the function will return the
#' "results" data frame, like hab_run().
#' # These two functions will produce the same results:
#'
#' hab_run_diagnostic(
#'   "fall_chinook", "Historical", "hist", pass_mort = FALSE)
#'
#' hab_run(
#'   "fall_chinook", "Historical", "hist", pass_mort = FALSE)
#'
#' @seealso
#' `hab_run()`
#' @export
#' @md
hab_run_diagnostic <- function(
  run_pop,
  run_scenario = "Current",
  run_era = "curr",
  beaver_model = "BIP",
  df_temperature = harp::temperature_df,
  df_width = harp::width_df,
  df_fl = harp::flowline_clean[harp::flowline_clean$pop == run_pop, ],
  pass_mort = TRUE,
  return_df = "results"){

  warning(
    "hab_run_diagnostic()` is intended for use in troubleshooting the habitat model")

  #run hab_run_internal
  hab_run_internal(
    run_pop = run_pop,
    run_scenario = run_scenario,
    run_era = run_era,
    beaver_model = beaver_model,
    df_temperature = df_temperature,
    df_width = df_width,
    df_fl = df_fl,
    pass_mort = pass_mort,
    return_df = return_df
  )

}

#' Run the habitat model (internal)
#'
#' Internal habitat model. Use \code{hab_run()} or \code{hab_run_diagnostic()} to
#' access the habitat model
#'
#' @param run_pop which pop to run
#' @param run_scenario which scenario to run.
#' @param run_era which era to run. The choices are hist, curr, mid, late.
#' @param df_temperature default is `harp::temperature_df`. Vary temps here
#' @param df_width default is `harp::width_df`. Vary widths here
#' @param df_fl all flowline reaches for the pop of interest
#' @param pass_mort should partial passage affect prespawn mortality?
#' @param return_df which data frame to return
#'
#'
hab_run_internal <- function(
  run_pop,
  run_scenario,
  run_era,
  beaver_model,
  df_temperature,
  df_width,
  df_fl,
  pass_mort,
  return_df = "results"
){

  # check inputs ---------------------------------------------------------------
  if (!(run_pop %in% pops_all)) {
    stop(sprintf(
      "'run_pop' is not one of: %s",
      paste0(sprintf("'%s'", pops_all), collapse = ", ")
    ))
  }
  if (!(run_scenario %in% scenarios_all)) {
    stop(sprintf(
      "'run_scenario' is not one of: %s",
      paste0(sprintf("'%s'", scenarios_all), collapse = ", ")
    ))
  }
  if (!(run_era %in% eras_all)) {
    stop(sprintf(
      "'run_era' is not one of: %s",
      paste0(sprintf("'%s'", eras_all), collapse = ", ")
    ))
  }
  if (!(beaver_model %in% c("BIP", "SP"))) {
    stop(sprintf(
      "'run_era' is not one of: %s",
      paste0(sprintf("'%s'", c("BIP", "SP")), collapse = ", ")
    ))
  }

  if (length(unique(df_fl$pop)) > 1) {
    stop("Multiple pops detected in input flowline.")
  }

  # calculate habitat areas-----------------------------------------------------
  # small rivers
  df_ss_clean <- hab_clean_ss(df_fl, run_scenario, run_era)

  df_ss <- hab_calc_ss_length(df_ss_clean, beaver_model)

  df_ss_area <- hab_calc_ss_area(df_width, df_ss)

  # floodplain habitats
  df_sch <- hab_create_side_channels(df_fl)

  df_fp_clean <- hab_clean_floodplain(df_sch, df_fl, run_scenario, run_era)

  df_fp <- hab_calc_fp_area(df_fp_clean)

  # large rivers
  df_lr <- hab_prep_lr(run_pop, df_fl)

  df_bw <- hab_clean_backwater(df_fl, df_sch, run_scenario, run_era)

  df_lr_clean <- hab_calc_lr_length(df_lr, run_scenario, run_era)

  df_w <- hab_apply_lr_woodmult(run_pop, df_lr_clean)

  df_lr_area <- hab_calc_lr_area(df_width, df_lr_clean, df_bw)
  # calc lifestage parameters --------------------------------------------------
  # rearing capacity
  df_cap <- hab_calc_capacity(
    run_pop,
    df_lr_area,
    df_ss_area,
    df_fp,
    df_w,
    df_temperature
  )

  # productivity
  df_prod <- hab_calc_productivity(run_pop, df_cap)

  # movement
  df_mvmt <- hab_calc_movement(run_pop, run_scenario, df_fp)

  # spawning (small stream habs - actually all streams under 30 m)
  df_spss <- hab_calc_capacity_spawn_ss(
    run_pop,
    df_fl,
    df_width,
    beaver_model,
    run_scenario,
    run_era
  )

  # spawning (floodplain habs)
  df_spfp <- hab_calc_capacity_spawn_fp(run_pop, df_fp_clean)

  # spawning (large river habs)
  df_splr_clean <- hab_calc_capacity_spawn_lr(
    df_fl,
    df_width,
    run_pop,
    run_scenario,
    run_era
  )

  # number of eggs
  df_cap_egg <- hab_calc_egg_cap(
    df_splr_clean,
    df_spss,
    df_spfp
  )

  # spawning (by subbasin)
  df_cap_spawn <- hab_calc_capacity_spawn(df_cap_egg, run_pop)

  # egg to fry survival
  df_prod_e2f <- hab_calc_productivity_egg2fry(df_fl, df_cap_egg)

  # prespawn survival
  df_prod_prespawn <- hab_calc_productivity_prespawn(
    run_pop,
    run_scenario,
    run_era,
    df_fl,
    df_cap_egg,
    df_temperature,
    pass_mort
  )

  # delta capacity
  df_cap_delta <- hab_calc_capacity_delta(run_pop, run_scenario, run_era)

  # clean up -------------------------------------------------------------------

  results <- hab_organize(
    run_pop,
    df_prod,
    df_cap_spawn,
    df_prod_prespawn,
    df_prod_e2f,
    df_mvmt,
    df_cap_delta
  )

  results <- tibble::add_column(results,
                                era = run_era,
                                .before = 'subbasin_num')

  return(get(return_df))
}
