#' Convert recurrence interval into egg to fry survival
#'
#' Equation derived in flow paper.
#'
#' @param ri recurrence interval
#'
#' @return
#' @noRd
calc_surv <- function(ri){
  log_surv <- -1.869 * ri^0.108
  exp(log_surv) # return survival
}

#' Flow to RI based on Pearson type III statistics
#' @param q_cfs flow in cubic feet per second
#' @importFrom smwrBase ppearsonIII

calc_RI <- function(q_cfs){

  # calculate exceedance probability for flow
  lp3_stat <- smwrBase::ppearsonIII(
    q = log10(q_cfs),
    mean =  logqmean,
    sd =  logqsd,
    skew = logqskew)

  1 / (1 - lp3_stat) # calc return interval
}


#' Convert q to surv
#'
#' @param q_cfs flow in cubic feet per second
#' @param era which era to run
#'
#' @noRd

calc_e2f_mult <- function(q_cfs, era = c("hist", "curr")){

  match.arg(era)

  ri <- calc_RI(q_cfs) # current era return interval
  if (era %in% c('hist', 'curr')) {
    q_mult <- 1 # no adjustment to current or historic flows
  }
  #else if (era == 'mid') { # adjust future flows
  #  q_mult <- 1.087 + 0.057 * log(ri)
  #}
  #else if (era == 'late') {
  #  q_mult <- 1.082 + 0.067 * log(ri)
  #}

  ri_adj <- calc_RI(q_cfs * q_mult) #adjusting flow according to era

  # Survival associated with the median flow (2-year recurrence interval)
  surv_median <- calc_surv(2) #0.133416

  # e2f multiplier is 1 + the percent change from the median condition
  1 + ((calc_surv(ri_adj) - surv_median) / surv_median)
}


#' Convert annual air temperature into water temperature adjustment value
#'
#' @param x annual air temperature
#'
#' @return annual temperature to be added to modeled reach temperature
#'
#' @noRd

calc_waterT_diff <- function(air_temp) {

  water_t <- b0 + b1 * air_temp # calc water temp with linear equation

  water_t - mean(water_t) # calc difference from time series mean water temp
}


#' Convert low flow into wetted width scalar
#'
#' Convert MARSS modeled low flow into width multiplier
#'
#' @param x annual low flow
#'
#' @return annual width multiplier
#'
#' @noRd
calc_width_mult <- function(x) {

  diff_from_mean <- (x - mean(x))/mean(x) # percent diff relative to mean
  width_diff <- diff_from_mean / 2 # width changes at half the rate of flow

  1 + width_diff
}

#' Run the habitat model with stochastic params
#'
#' Use the MARSS simulation to generate annual variation in
#' temperature, wetted width and peak flow. This function calculates those
#' annual variations and brings them into the habitat model.
#'
#' Each param has a different way of being applied
#' q_low -- rearing capacity -- scalar * wetted width -- reach level
#' q_peak -- incubation -- scalar * egg_fry_surv -- on subbasin level params
#' t_peak -- rearing capacity and survival-- scalar + temperature -- reach level
#'
#' Note the output is automatically run through lcm_assign().
#'
#' @param pop which population to run
#' @param hab_scenario which scenario to run
#' @param era which era to run? curr, mid, late
#' @param runs How many runs/iterations? vector of values, e.g. 1:10
#' @param years How many years? Vector of values, e.g. 1:110 (default)
#' @param print_status Print model progress to console?
#'
#' @export
hab_run_stochastic <- function(pop,
                               hab_scenario,
                               era,
                               beav_model = "BIP",
                               runs = 1,
                               years = 1:100,
                               print_status = FALSE) {

  # calculate temp diff, width mult and e2f adj for years/runs
  df_marss_wide <- harp::df_marss %>%
    dplyr::mutate(dplyr::across(c(year, run), as.numeric)) %>%
    dplyr::filter(year %in% years, run %in% runs) %>%
    tidyr::pivot_wider(names_from = var, values_from = value) %>%
    dplyr::mutate(
      temp_diff_i = calc_waterT_diff(t_peak),
      width_mult_i = calc_width_mult(q_low),
      e2f_i = dplyr::case_when(
        pop == 'winter_steelhead' ~ calc_e2f_mult(q_peak_win_sthd, era),
        pop == 'summer_steelhead' ~ calc_e2f_mult(q_peak_sum_sthd, era),
        TRUE ~ calc_e2f_mult(q_peak, era))) %>%
    dplyr::select(run, year, temp_diff_i, width_mult_i, e2f_i)

  scenarios <- tidyr::crossing( # create all combinations of inputs
    pop,
    scenario = hab_scenario,
    era,
    df_marss_wide)

  purrr::pmap_dfr( # for each row, adjust flowlines and run habitat model
    scenarios,
    function(pop, scenario, era, run, year, temp_diff_i, width_mult_i, e2f_i){

      # adjust temperatures
      temp_i <- dplyr::mutate(
        harp::temperature_df,
        dplyr::across(c('temp_7DADM', 'temp_prespawn'), ~.x + temp_diff_i),
        temp_mdm = temp_Jn121 + hab_mwmt_to_mdm(temp_diff_i))

      # adjust widths
      width_i <- dplyr::mutate(
        harp::width_df,
        width_s = width_s * width_mult_i)

      h <- suppressMessages( # run hab model
        hab_run(
          pop,
          hab_scenario,
          era,
          beav_model,
          temp_i,
          width_i))


      h_scaled <- dplyr::mutate( # adjust egg to fry survival
        h,
        value = ifelse(param == 'eggtofry_surv', value * e2f_i, value),
        run = as.numeric(run),
        year = year
        #year = as.numeric(year)
        )

      if(print_status) {message(paste('Run', run, '-- Year', year))}

      return(h_scaled)

    })
}
