
#' @export
plot.harp_lcm <- function(x, target_ls = "spawners"){

  if(!(target_ls %in% unique(x$lifestage))){
    stop(sprintf(
      "No \"%s\" lifestage in model output. Available lifestages are: \n %s",
      target_ls,
      paste0(sprintf("'%s'", unique(x$lifestage)), collapse = ", ")))
  }

  if(nrow(dplyr::distinct(x, pop, era, scenario)) > 1){
    stop("Multiple populations, eras, and/or scenarios detected")
  }

  # formatting function
  round_commas <- function(x) scales::comma_format()(round(x))

  basin_sum <- x %>% # sum fish by year and run
    dplyr::filter(lifestage == target_ls) %>%
    dplyr::group_by(across(any_of(c('year', 'run')))) %>% # possible grouping fields
    dplyr::summarize(n = sum(n), .groups = "keep")

  basin_stats <- basin_sum %>% # summarize by year
    dplyr::group_by(year) %>%
    dplyr::summarize(
      median_n = median(n),
      max_n = max(n), # these are redundant in deterministic model
      min_n = min(n))

  #stats for plot
  basin_range <- round_commas(max(basin_sum$n)-min(basin_sum$n))
  basin_median <- round_commas(median(basin_sum$n))
  ylabel <- ifelse(target_ls == "spawners", "Spawners", target_ls)

  #optional multirun layer
  multiline <- if("run" %in% names(basin_sum)){
    ggplot2::geom_line(
      data = basin_sum,
      ggplot2::aes(year, n, group = run),
      color = "#cccccc",
      size = 0.1)
  } else {NULL}

  basin_stats %>%
    ggplot2::ggplot()+
    multiline+ # either lines for individual runs or nothing
    ggplot2::geom_hline(
      yintercept = median(basin_sum$n),
      color = "blue", linetype = 2)+
    ggplot2::geom_line(ggplot2::aes(year, median_n))+
    ggplot2::scale_x_continuous(
      #limits = c(min(basin_sum$year), max(basin_sum$year)), # don't show year 0
      expand = c(0, 0))+
    ggplot2::theme_minimal()+
    ggplot2::annotate(
      "text",
      label = sprintf(
        "range = %s, median = %s", basin_range, basin_median),
      x = min(basin_sum$year), y = max(basin_sum$n), vjust = 0, hjust = 0)+
    ggplot2::labs(x = "Year", y = ylabel)
}


#' @export
print.harp_lcm <- function(x){
  add_commas <- function(x) scales::comma_format()(x)
  roundmed <- function(x) add_commas(round(median(x))) # custom func for disp
  lifestages <- unique(x$lifestage) # see what life stages were exported

  grouping_fields <- c('pop', 'era', 'scenario', 'year', 'run', 'lifestage')

  basin_sum <- x %>%
    dplyr::group_by(across(any_of({{grouping_fields}}))) %>%
    dplyr::summarize(n = sum(n), .groups = "keep") %>%
    dplyr::group_by(pop, era, scenario, lifestage) %>%
    dplyr::summarize(n = roundmed(n), .groups = "keep") %>%
    tidyr::pivot_wider(names_from = lifestage, values_from = n) %>%
    as.data.frame() # convert to df for compact printing

  cat("Summary:\n")
  print(basin_sum) # print summary lines
  cat("\nPreview:\n")
  print(tidyr::as_tibble(x)) # use tibble method to print df
}
