#' Assign steelhead habitat parameters
#'
#' A wrapper around lcm_assign() that takes in two sets of steelhead habitat
#' parameter - one for the winter run and another for the summer run. It assigns
#' separate parameters for each run's spawning lifestage and combines rearing
#' parameters
#'
#' @param steelhead_winter_params dataframe output of hab_run()
#' @param steelhead_summer_params dataframe output of hab_run()
#' @export

lcm_assign_steelhead <- function(
  steelhead_winter_params,
  steelhead_summer_params){

  if(!all(
    unique(steelhead_winter_params$pop) == "winter_steelhead",
    unique(steelhead_summer_params$pop) == "summer_steelhead"
  )){stop("Habitat data not entered correctly. Check documentation.")}

  wparams <- steelhead_winter_params %>%
    dplyr::mutate(pop = "steelhead") %>%
    lcm_assign() %>%
    dplyr::rename( # rename spawning lifestages to separate between runs
      egg_cap_w = egg_cap,
      pop_init_w = pop_init,
      egg_fry_surv_w = egg_fry_surv,
      S_up_w = S_up) %>%
    dplyr::mutate(pcap_w = parr_cap) # add marker for winter-run capacity

  sparams <- steelhead_summer_params %>%
    dplyr::mutate(pop = "steelhead") %>%
    lcm_assign() %>%
    dplyr::rename(
      egg_cap_s = egg_cap,
      pop_init_s = pop_init,
      egg_fry_surv_s = egg_fry_surv,
      S_up_s = S_up) %>%
    dplyr::mutate(pcap_s = parr_cap) # add marker for summer-run capacity

  dplyr::bind_rows(wparams, sparams) %>% # combine both species
    dplyr::group_by(
      dplyr::across(dplyr::any_of( # one row for each subbasin-year pair
        c("subbasin_num", "subbasin", "year")))) %>%
    dplyr::summarize(
      dplyr::across(.fns = max, na.rm = T)) %>% # use maximum of shared params
    dplyr::mutate( # don't allow spawning for winter-run fish in subbasins where
      # they have less than 1/4 of the summer-run spawning capacity
      egg_cap_w = ifelse(pcap_w * 4 < pcap_s, 0, egg_cap_w),
      egg_cap_s = ifelse(pcap_s * 4 < pcap_w, 0, egg_cap_s)) %>%
    dplyr::arrange(as.numeric(subbasin_num))
}
