
#' Assign habitat parameters
#'
#' Rearrange the habitat parameters into wide format and add lcm parameters (e.g. params_coho).
#' Species specific cleanup actions taken.
#'
#' @param hab_scenario_params dataframe output of hab_run()
#' @export

lcm_assign <- function(hab_scenario_params){

  run_pop <- unique(hab_scenario_params$pop)

  if(run_pop %in% c("winter_steelhead", "summer_steelhead")){
    stop("Use lcm_assign_steelhead() for summer or winter steelhead runs")
  }

  # Create df of hab params and subbasin
  # move to habitat model
  dat_df <- dplyr::left_join(
    hab_scenario_params, subbasin_names, by = "subbasin_num") %>%
    dplyr::mutate(subbasin = as.factor(noaa_subba),
                  pop = run_pop) %>%
    dplyr::arrange(as.numeric(subbasin_num)) # sort by subb number

  # Convert to wide format
  # and append fixed variables from data-raw/params/params_pop.R
  dat_df <- dat_df %>%
    tidyr::pivot_wider(names_from = param, values_from = value) %>%
    append(get(paste0('params_', run_pop))) %>%
    data.frame() %>% # convert back to df
    replace(is.na(.), 0)  # remove NA and NaN

  # columns to group by if present
  group_vars <- c("scenario", "era", "run", "year")

  # Initial population per subbasin
  # These are overwritten with Neq in lcm_run_iteratively()
  dat_df <- dat_df %>%
    dplyr::group_by(across(any_of(group_vars))) %>%
    dplyr::mutate(pop_init = pop_init * (eggs/sum(eggs))) %>%
    dplyr::ungroup() %>%
    dplyr::rename(
      egg_cap = eggs,
      egg_fry_surv = eggtofry_surv,
      S_up = prespawn_surv
      )

  if (run_pop == "coho") {
    # adjust values for coho
    dat_df <- dat_df %>%
      dplyr::mutate(movement = movement/100) %>%
      dplyr::rename(parr_surv = surv_s,
                    parr_cap = capacity_s,
                    parr_smolt_surv = surv_w,
                    parr_smolt_cap = capacity_w,
                    redist = movement)
  } #end if coho


  if (run_pop == "fall_chinook") {

    dat_df <- dat_df %>%
      dplyr::mutate(
        colnz_surv = surv_s^(1/12),# 1 week of freshwater mortality
        # Weekly productivity scaled with June temperatures
        # This is weighted so that 34% of fish will get the impact of warm June
        # temperatures for 3 of 11 weeks of the spring rearing period
        spring_surv = 0.66*surv_s^(11/12)+0.34*surv_s^(8/12)*surv_s_t^(3/12),
        ) %>%
      dplyr::select(-c(surv_s, surv_s_t, capacity_s_t)) %>%
      dplyr::rename(
        spring_cap = capacity_s,
        summer_cap = capacity_s_2,
        winter_surv = surv_w,
        summer_surv = surv_s_2)
  } # end if chinook

  if (run_pop == "steelhead") {

    dat_df <- dplyr::rename(
      dat_df,
      parr_surv = surv_s,
      parr_cap = capacity_s,
      first_winter_surv = surv_w,
      first_winter_cap = capacity_w,
      second_summer_cap  = capacity_s_2,
      second_summer_surv = surv_s_2,
      second_winter_cap  = capacity_w_2,
      second_winter_surv = surv_w_2
    )
  }

  if (run_pop == 'chum') {

    dat_df <- dplyr::rename(
      dat_df,
      fry_colonization_surv = fry_colonization_surv,
      fry_colonization_cap = fry_colonization_cap,
      fry_migrant_surv = surv_s
    )
  }

  dat_df

}
