#' Chinook LCM
#'
#' Define the life cycle model for chinook. Lifestages are:
#' \enumerate{
#' \item  Spawners
#' \item  Eggs
#' \item  Fry
#' \item  Parr migrants
#' \item  Fry migrants
#' \item  Ocean0
#' \item  Ocean1
#' \item  Ocean2
#' \item  Ocean3
#' \item  Ocean4
#' \item  Ocean5
#' \item  Total Run
#' }
#'
#' Currently works for coho, Chinook, steelhead or chum
#'
#' @param mat_in N matrix
#' @param mat_out blank N matrix
#' @param hab_params param list
#'
#' @export
lcm_chinook <- function(mat_in, mat_out, hab_params, yr_label) {

  list2env(hab_params, envir = environment())

  NOR_total <- diag(mat_in['spawners',,])

  eggs <- BH_func(S = NOR_total*sex_ratio, p = fecund, c = egg_cap)

  pre_fry <- eggs * egg_fry_surv

  # store values
  diag(mat_out['eggs',, ]) <- eggs
  diag(mat_out['pre_fry',, ]) <- pre_fry

  # subyearlings that migrate after 12 weeks
  # week #1
  natal_fry <- BH_func(S = pre_fry, p = colnz_surv, c = spring_cap)
  #weeks #2-12
  natal_fry_distrib <- distrib_fish(natal_fry, mat_move_downstream, 1)
  parr <- BH_func_distrib( # these are conditions in the rearing basin
    natal_fry_distrib, spring_surv, spring_cap)
  parr_migrants <- parr * subyr_ratio

  # subyearlings that migrate after 1 week
  fry_migrants <- ((pre_fry * colnz_surv) - natal_fry)*fry_migr_prod

  # fish that attempt to become yearlings
  stayers <- parr * (1-subyr_ratio) # store matrix
  summer_survivors <- BH_func_distrib(
    stayers, summer_surv, summer_cap)

  # store values
  diag(mat_out['natal_fry',, ]) <- natal_fry
  diag(mat_out['natal_fry_distrib',, ]) <- colSums(natal_fry_distrib)
  mat_out['parr',, ] <- parr
  mat_out['parr_migrants',, ] <- parr_migrants
  mat_out['stayers',,] <- stayers
  mat_out['summer_survivors',,] <- summer_survivors
  diag(mat_out['fry_migrants',, ]) <- fry_migrants

  # YEAR 2 ---------------------------------------------------------------------
  # bring in survivors from previous year
  # then multiply by the winter survival of the REARING subbasin
  # yearlings outmigrate in the spring

  yearlings <- t(t(mat_in['summer_survivors',,]) * winter_surv)

  mat_out['yearlings',,] <- yearlings

  # ESTUARY --------------------------------------------------------------------
  # Apply bay survival (ds migration, delta, bay, nearshore productivity)

  # split fry between delta and nearshore
  # combine all fry in delta, then reassign the survivors (proportional to input)
  delta_fry <- BH_func(sum(fry_migrants), delta_prod[1], capacity_delta[1])*
    prop.table(fry_migrants)
  diag(mat_out['delta_fry',,]) <- delta_fry
  diag(mat_out['nearshore_fry',,]) <- fry_migrants - delta_fry


  # Ocean survival, subyearlings
  diag(mat_out['ocean0_0',, ]) <- (delta_fry + rowSums(parr_migrants)) * ps_parr_surv
  mat_out['ocean0_1',, ] <- mat_in['ocean0_0',, ] * so1 # 2 year olds, ocean1
  mat_out['ocean0_2',, ] <- mat_in['ocean0_1',, ] * (1 - b0_2) * so2
  mat_out['ocean0_3',, ] <- mat_in['ocean0_2',, ] * (1 - b0_3) * so3
  mat_out['ocean0_4',, ] <- mat_in['ocean0_3',, ] * (1 - b0_4) * so4

  # ocean survival, yearlings
  mat_out['ocean1_0',, ] <- rowSums(yearlings) * ps_yr_surv
  mat_out['ocean1_1',, ] <- mat_in['ocean1_0',, ] * so1
  mat_out['ocean1_2',, ] <- mat_in['ocean1_1',, ] * (1 - b1_2) * so2
  mat_out['ocean1_3',, ] <- mat_in['ocean1_2',, ] * (1 - b1_3) * so3

  mat_out['total_run_subyr',,] <- #Total run (subyearling migrants)
    b0_2 * mat_in['ocean0_1',, ] +
    b0_3 * mat_in['ocean0_2',, ] +
    b0_4 * mat_in['ocean0_3',, ] +
    mat_in['ocean0_4',,]

  mat_out['total_run_yr',,] <- #Total run (yearling migrants)
    b1_2 * mat_in['ocean1_1',, ] +
    b1_3 * mat_in['ocean1_2',, ] +
    mat_in['ocean1_3',, ]

  mat_out['total_run',,] <- # both
    mat_out['total_run_subyr',,] +
    mat_out['total_run_yr',,]

  total_run <- diag(mat_out['total_run',,])

  # collect broodstock proportionally from subbasins based on run size
  # move 140 to raw-data later
  # move year into hab params to improve warning message

  tot_bs <- unique(tot_bs) # make vector length 1
  broodstock <- prop.table(total_run)*tot_bs # divide broodstock proportionally
  if(sum(total_run, na.rm = T) < tot_bs & sum(total_run, na.rm = T) > 0){
    warning(sprintf("No broodstock removed in %s, only %s fish",
                    yr_label,
                    round(sum(total_run, na.rm = T))))
    diag(mat_out['spawners',,]) <- total_run * S_up * (1 - hr)
  } else{
    diag(mat_out['spawners',,]) <- (total_run - broodstock) *
      S_up * (1 - hr) # Spawners
    }

  return(mat_out)

} # end chinook
