#' Coho LCM
#'
#' Define the life cycle model for coho. Lifestages are:
#' \enumerate{
#' \item  Spawners
#' \item  Eggs
#' \item  Pre Fry
#' \item  Fry
#' \item  Spring redistribution (short)
#' \item  Parr
#' \item  Fall redistribution (spread across mainstem)
#' \item  Smolts
#' \item  Ocean0
#' \item  Ocean1
#' \item  Ocean2
#' \item  Total Run
#' }
#'
#'
#' @param mat_in N matrix
#' @param mat_out blank N matrix
#' @param hab_params param list
#'
#' @export
lcm_coho <- function(mat_in, mat_out, hab_params, yr_label) {

  list2env(hab_params, envir = environment())

  NOR_total <- mat_in['spawners', ]

  eggs <- BH_func(S = NOR_total * sex_ratio, p = fecund, c = egg_cap)
  pre_fry <- eggs * egg_fry_surv

  fry <- pre_fry * fry_colonization

  # Spring distribution
  fry_distributed <- distrib_fish(fry, mat_move_nearest_ms_only, percent_fry_migrants)

  parr <- BH_func_distrib(fry_distributed, parr_surv, parr_cap)

  # Fall distribution
  parr_distributed <- distrib_fish(parr, mat_move_downstream, redist)

  smolts <- BH_func_distrib(parr_distributed, parr_smolt_surv, parr_smolt_cap)

  # Ocean stages
  # rowSums sums fish that originate in a given subbasin
  mat_out['ocean0', ] <- rowSums(smolts) * bay # ocean0
  mat_out['ocean1', ] <- mat_in['ocean0', ] * so # ocean1
  mat_out['ocean2', ] <- mat_in['ocean1', ] * (1 - b2) * so # ocean2, 'spawners' pre harvest

  # Other stored stages
  mat_out['eggs', ] <- eggs
  mat_out['pre_fry', ] <- pre_fry
  mat_out['fry', ] <- fry
  mat_out['parr', ] <- rowSums(parr) # parr originating in a subbasin
  mat_out['smolts', ] <- rowSums(smolts)
  mat_out['total_run', ] <- (b2*mat_in['ocean1', ] + mat_in['ocean2', ]) # Total run
  mat_out['spawners', ] <- mat_out['total_run', ] * S_up * (1 - hr) #spawners

  mat_out['fry_movers', ] <- rowSums(fry_distributed) - diag(fry_distributed)
  mat_out['parr_movers', ] <- rowSums(parr_distributed) - diag(parr_distributed)
  mat_out['natal_smolts', ] <- diag(smolts)
  mat_out['non_natal_smolts', ] <- rowSums(smolts) - diag(smolts)

  return(mat_out)

}
