# LCM functions

#' Geometric mean
#'
#' Calculate the geometric mean of a vector.
#'
#' Numbers less than or equal to 0 are disregarded.
#' @param x Vector of numbers
#' @examples
#' geo_mean(1:10)
#' geo_mean(-10:10)
#' @export
geo_mean <- function(x){
  x[which(x <= 0)] <- NA
  exp(mean(log(x), na.rm = TRUE))
}


#' Beverton-Holt function
#'
#' The Beverton-Holt function in the form
#' \deqn{N_{s+1} = \frac{N_s p}{1 + \frac{p}{c}N_s}}
#'
#' @param S input life stage abundance
#' @param p productivity or density independent survival from the input stage to
#'  the output stage.
#' @param c capacity of the output lifestage
#'
#' @examples
#' BH_func(1000, 0.5, 800)
#' @export
BH_func <- function(S, p, c){
  # generic Beverton-Holt
  # S = parental state
  # p = productivity or survival
  # c = capacity
  #p[which(p == 0)] <- 0.001
  c[which(c == 0)] <- 1
  # S[which(S == 0)] <- 1
  recruits <- (S  * p) / (1 + (p / c) * S)
  recruits[recruits < 1] <- 0 # cleans up basins with less than 1 fish
  recruits
}


#' Distribute fish between basins
#'
#' Move fish from natal basin to rearing basin. The rules are set by \code{move_mat}.
#' Note there is currently no way to move fish from rearing basin to another rearing basin.
#'
#' @param n_in Either a vector of abundance or matrix output of this function
#' @param move_mat Matrix to control movement of fish in and out of basins.
#' origin basins are represented by matrix rows and destination basins are
#' represented by matrix columns. The matrix should have values at origin-
#' destination pairs where movement is allowed and 0s otherwise. Rows should
#' sum to 0 or 1.
#' @param move_prcnt What percent of fish will move to each destination basin.
#' When given as a vector, each vector element corresponds with an origin basin,
#' so a vector c(0.05, ....) would move 5% of the fish from the first origin
#' basin to each of its allowed output basin.
#'
#' @return A matrix of abundance with natal basin rows and rearing basin columns
#' @export
distrib_fish <- function(n_in, move_mat, move_prcnt){

  if (any(!(rowSums(move_mat) %in% c(0,1)))){
    stop('matrix row sums must be 0 or 1')
  }
  if (any(move_prcnt > 1)) stop('move_prcnt must be <= 1')

  if (is.matrix(n_in)) {
    n <- colSums(n_in) # n fish currently in basin
    # turn move_prct into vector if it isn't already
    if(length(move_prcnt) == 1){
      move_prcnt <- rep(move_prcnt, nrow(move_mat))
    }
  } else {
    n <- n_in
  } # vector: number of fish

  move_mat <- move_mat * move_prcnt # percent of fish in destination basins

  if (is.vector(n_in)){
    # replaces the diagonal with the % of fish that haven't left the basin
    # (fish remaining in origin basins)
    diag(move_mat) <- 1 - rowSums(move_mat) + diag(move_mat)
  }

  n_out_basin <- n * move_mat # multiply percents by # of fish


  if (is.matrix(n_in)) {
    # column multiplication via matrix multiplication
    # movement is divided proportionally between fish who migrated into
    # a basin and fish who were born there
    stayers <- n_in %*% diag(1-move_prcnt) # how many fish don't migrate?

    props <- prop.table(n_in, 2) # where did fish come from originally?
    props[is.nan(props)] <- 0 # remove NaNs
    movers <- props %*% n_out_basin # reallocate input basins based on natal

    n_out_basin <- movers + stayers # add stayers to movers
  }

  return(n_out_basin)
}

#' BH function for distributed fish
#'
#' A light wrapper of the \code{\link{BH_func}}, which takes a matrix as
#' an input rather than a vector. Sums the fish rearing in a given basin
#' and uses that summed abundance to run \code{BH_func}.
#'
#' @param mat_move_fish Output of \code{\link{distrib_fish}}
#' @param p productivity
#' @param c capacity
#'
#' @return A matrix of values
#' @export
BH_func_distrib <- function(mat_move_fish, p, c){
  n_in_basin <- colSums(mat_move_fish)

  n_out_basin <- BH_func(n_in_basin, p, c)

  n <- sweep(prop.table(mat_move_fish, 2), 2, n_out_basin, `*`)
  n[is.nan(n)] <- 0

  n
}

#' Element-wise vector division with replacement of NAs with zeroes
#'
#' Divide vectors and replace any resulting NAs with 0s
#'
#' @param x numerator
#' @param y denominator
#'
#' @return A vector of values
#' @export
safe_divide <- function(x, y){
  z <- x/y
  z[is.na(z)] <- 0
  return(z)
}





