

#' Initialize arrays
#'
#' Returns two arrays: \code{mat} and \code{one_run}
#' No need to run this independently. Run internally in \code{lcm_run}
#'
#' @param hab_params_df output of \code{\link{hab_run}} or \code{\link{hab_run_stochastic}}
#'
#' @export


lcm_initialize <- function(hab_params_df) {

  pop <- unique(hab_params_df$pop)

  if (length(pop) != 1) stop('Only one pop at a time to lcm_initialize')

  # Gather subbasin names
  subbasin_name <- unique(hab_params_df$subbasin)
  num_subbasin <- length(subbasin_name)

  if ('year' %in% colnames(hab_params_df)) {
    years <- unique(hab_params_df$year)

  } else {
    years <- 1:100

  }


  # Coho lifestages and sensitivity parameters
  if (pop == "coho") {
    lifestages <- c(
      'ocean0',
      'ocean1',
      'ocean2',
      'total_run',
      'spawners',
      # Above here is needed for spawner matrix
      'eggs',# Here and below are stored for diagnostics
      'pre_fry',
      'fry',
      'parr',
      'smolts',
      'fry_movers',
      'parr_movers',
      'natal_smolts',
      'non_natal_smolts'
    )

    sens.params <- c(
      "Egg.capacity",
      "Incubation.productivity",
      "Summer.rearing.productivity",
      "Summer.rearing.capacity",
      "Winter.rearing.productivity",
      "Winter.rearing.capacity",
      "Prespawn.productivity",
      "geomean")# model output
  } #end if coho




  if (grepl('chinook', pop)) {
    # either spring_chinook or fall_chinook

    lifestages <- c(
      'eggs',
      'pre_fry',
      'natal_fry',
      'fry_migrants',
      'natal_fry_distrib',
      'parr',
      'parr_migrants',
      'delta_fry',
      'nearshore_fry',
      'stayers',
      'summer_survivors',
      "yearlings",
      sprintf("ocean0_%s", 0:4), # 4 ocean ages for subyearlings
      sprintf("ocean1_%s", 0:3), # 3 ocean ages for yearlings
      'total_run_yr',
      'total_run_subyr',
      'total_run',
      'spawners'
    )


    sens.params <- c(
      "Egg.capacity",
      "Incubation.productivity",
      "Rearing.productivity",
      "Rearing.capacity",
      "Rearing.productivity.June",
      "Prespawn.productivity",
      "geomean")

  } # end if chinook



  if (pop == "steelhead") {
    runs <- c("summer", "winter") # names of runs
    fw_ages <- paste0("age", 1:3) # allowed freshwater ages
    combo_ages <- c( # allowed age combinations
      sprintf("age%s_%s", 1, 1:3),
      sprintf("age%s_%s", 2, 1:3),
      sprintf("age%s_%s", 3, 1:2)
    )

    lifestages <- c(
      paste("total_run", runs, sep = "_"),
      paste("kelt", runs, sep = "_"),
      paste("new_spawners", runs, sep = "_"),
      "spawners", # sum of both runs
      paste("spawners", runs, sep = "_"),
      paste("eggs", runs, sep = "_"),
      paste("pre_fry", runs, sep = "_"),
      "parr",
      paste("smolts", runs, sep = "_"),
      purrr::pmap_chr( # this function creates buckets for stayers and smolts
        tidyr::expand_grid( # age3_stayers will remain unused
          fw_ages,
          stage = c("stayers", "smolts"),
          runs),
        ~sprintf("%s_%s_%s", ..1, ..2, ..3)),
      purrr::pmap_chr( # this function creates buckets for ocean and spawners
        tidyr::expand_grid( # use all combinations of ages, stages, and runs
          combo_ages,
          stage = c("ocean", "spawn"),
          runs),
        ~sprintf("%s_%s_%s", ..1, ..2, ..3)) # paste them together as x_y_z
    )

    sens.params <- c(
      "Egg.capacity",
      "Incubation.productivity",
      "First.summer.productivity",
      "First.summer.capacity",
      "First.winter.productivity",
      "First.winter.capacity",
      "Second.summer.productivity",
      "Second.summer.capacity",
      "Second.winter.productivity",
      "Second.winter.capacity",
      "Prespawn.productivity",
      "geomean")# model output
  } #end if steelhead



  # Create arrays to hold fish lifestages by spatial units ----

  if (pop == "coho") { # 2D matrix for coho
    N <- matrix(
      0,
      nrow = length(lifestages),
      ncol = num_subbasin,
      dimnames = list(lifestages = lifestages, subbasin = subbasin_name)
    )
  } else { # 3D array for other populations
    N <- array(
      0,
      c(length(lifestages), num_subbasin, num_subbasin),
      dimnames = list(
        lifestage = lifestages,
        subbasin = subbasin_name, # natal basins
        rearing_basin = subbasin_name))
  }


  # Store all model results in 3 dimensional array: years x lifestage x subbasin
  one_run <- array(
    NA,
    c(
      length(years),
      length(lifestages),
      num_subbasin
    ),
    dimnames = list(year = years,
                    lifestage = lifestages,
                    subbasin = subbasin_name)
  )


  # return list
  array_list <- list(mat = N,
                     one_run = one_run)

  return(array_list)

}

