#' Run the LCM
#'
#' Run a given life cycle model with a given set of parameters.
#'
#' @param hab_params Dataframe with parameters as columns and subbasins
#' as rows, typically output of \code{\link{lcm_assign}}. Optional to have a column for year.
#' @param output_lifestages which lifestages to output. 'spawners' is default,
#' but any lifestages can be used. If you want to output all lifestages, use 'all'.
#'
#' @export
lcm_run <- function(hab_params, output_lifestages = 'spawners'){

  if ('run' %in% colnames(hab_params)) { # for stochastic hab data
    runs <- unique(hab_params$run)
    purrr::map_dfr(
      runs, # go through each run in data frame
      ~ lcm_run_internal(subset(hab_params,run == .x), output_lifestages))
  } else {lcm_run_internal(hab_params, output_lifestages)}

}



#' Run the LCM for a single set of consecutive years
#'
#' Run a given life cycle model with a given set of parameters.
#'
#' @param hab_params Dataframe with parameters as columns and subbasins
#' as rows, typically output of \code{\link{lcm_assign}}. Optional to have a column for year.
#' @param output_lifestages which lifestages to output. 'spawners' is default,
#' but any lifestages can be used. If you want to output all lifestages, use 'all'.
#'
#' @noRd
lcm_run_internal <- function(hab_params, output_lifestages = 'spawners'){

  # TEMPORAL EXTENT ------------------------------------------------------------
  # use years from hab_params or default to 100
  if ('year' %in% colnames(hab_params)) {
    run_years <- max(as.numeric(hab_params$year))

  } else {
    run_years <- 100
  }

  # HABITAT PARAMETERS ---------------------------------------------------------
  # helper function to get habitat parameters for a given year
  create_param_yeari <- function(year_subset) {
    # If hab_params has a column named year, create list with one set of params
    # for year. Useful if the conditions vary over time. Otherwise, collect
    # the fixed parameters
    if ('year' %in% colnames(hab_params)) {
      as.list(subset(hab_params, year == year_subset))
    } else {
      hab_params
    }
  }

  # POPULATION STRUCTURE -------------------------------------------------------
  # create list of two arrays to hold results for each year, subbasin and life
  # stage
  initialized_arrays <- lcm_initialize(hab_params)
  # load the initialized arrays" "mat" and "one_run" into the environment.
  list2env(initialized_arrays, envir = environment())
  N <- mat # N is the "current state" population structure matrix that will be
  # recursively edited. "mat" will stay intact as a template

  # MODEL SELECTION ------------------------------------------------------------
  # what species are we running?
  pop <- unique(hab_params$pop)
  is_steelhead <- (pop == "steelhead") # different seeding & burn-in for sthd
  burn_years <- if(is_steelhead) {150} else {50} # longer burn-in for steelhead

  if(run_years < burn_years & 'run' %in% colnames(hab_params)){
    stop(sprintf(
      "At least %s years required for %s burn-in for %s",
      burn_years, pop))
  }

  # select species-specific model to use in run.
  if (grepl('chinook', pop)) {
    model <- lcm_chinook
  } else {
    model <- get(paste0('lcm_', pop))
  }

  # Define function generator based on the species-specific model
  run_model <- function(mat_in, year_subset = NA, yr_label = NA) {
    model(mat_in = mat_in,
          mat_out = mat, # this does NOT modify `mat` within lcm-run's env
          hab_params = create_param_yeari(year_subset),
          yr_label = yr_label)
  }

  # RUN THE MODEL --------------------------------------------------------------

  # Run 10 years to fill initialize matrix
  # Use year 1 conditions for each of the 10 years
  for (i in 1:10) {
    if(is_steelhead){ # seed runs seperately for steelhead
      diag(N['new_spawners_summer',,]) <- create_param_yeari(1)$pop_init_s
      diag(N['new_spawners_winter',,]) <- create_param_yeari(1)$pop_init_w
      N <- run_model(N, 1, sprintf("initialization year %s", i))
    } else if(pop == "fall_chinook"){ # chinook need 3D initialization
      diag(N['spawners',, ]) <- create_param_yeari(1)$pop_init
      N <- run_model(N, 1, sprintf("initialization year %s", i))
    } else {
      N['spawners', ] <- create_param_yeari(1)$pop_init
      N <- run_model(N, 1, sprintf("initialization year %s", i))
    }
  }# End initialize


  # Run 50-150 years to burn-in
  # Use stochastic values, when available for burn-in
  if('run' %in% colnames(hab_params)){
    for (i in 1:burn_years) {
      N <- run_model(N, i, sprintf("burn-in year %s", i))
    }
  } else{
    for (i in 1:burn_years) {
      N <- run_model(N, 1, sprintf("burn-in year %s", i))
    }
  }# End initialize

  for (i in 1:run_years) {
    if(pop %in% c("steelhead", "fall_chinook")){ # these pops use 3d array
      # Store the outputs of this loop
      N <- run_model(N, i, sprintf("model year %s", i))
      one_run[i,,] <- rowSums(N, dims = 2) # store everything in the natal basin
    } else{
      # Store the outputs of this loop
      N <- run_model(N, i, sprintf("model year %s", i))
      one_run[i,,] <- N
    }
  }

  # CLEAN UP THE RESULTS -------------------------------------------------------

  out_df <- as.data.frame.table(one_run) %>%
    dplyr::rename(n = Freq) %>%
    tibble::add_column(
      pop = pop,
      scenario = unique(hab_params$scenario),
      era = unique(hab_params$era),
      .before = 1) %>%
    dplyr::mutate(year = as.numeric(year))

  if ('run' %in% colnames(hab_params)) {
    out_df <- tibble::add_column(
      out_df,
      run = unique(hab_params$run),
      .after = 'era')
  }

  if (!'all' %in% output_lifestages){
    out_df <- subset(out_df, lifestage %in% output_lifestages)
  }

  class(out_df) <- c("harp_lcm", class(out_df))

  return(out_df)

}


