#' Steelhead LCM
#'
#'Define the life cycle model for steelhead. Lifestages are:
#' \enumerate{
#' \item  Spawners (first time and kelts)
#' \item  Eggs
#' \item  Pre-fry
#' \item  Parr
#' \item  Parr migrants
#' \item  Ocean stages 1.1, 1.2, 2.1, 2.2, 2.3, 3.1, 3.2
#' \item  Total Run
#' }
#'
#' @param mat_in N matrix
#' @param mat_out blank N matrix
#' @param hab_params param list
#'
#' @export
lcm_steelhead <- function(mat_in, mat_out, hab_params, yr_label) {

  list2env(hab_params, envir = environment())

  # summer and winter runs are considered not to compete during spawning
  # count first-time spawners
  # count incoming females
  NOR_first_spawn_w <- diag(mat_in["new_spawners_winter",,])* sex_ratio
  NOR_first_spawn_s <- diag(mat_in["new_spawners_summer",,])* sex_ratio

  # count repeat spawners, all assumed to be female
  NOR_kelt_w <- diag(mat_in["kelt_winter",,])
  NOR_kelt_s <- diag(mat_in["kelt_summer",,])

  # total females
  NOR_total_w <- NOR_first_spawn_w + NOR_kelt_w
  NOR_total_s <- NOR_first_spawn_s + NOR_kelt_s

  # percent of each type of female
  prct_firstspawn_w <- NOR_first_spawn_w / NOR_total_w
  prct_firstspawn_s <- NOR_first_spawn_s / NOR_total_s

  # Weighted fecundity
  fecund_w <- fecund_first * prct_firstspawn_w +
    fecund_respawn * (1-prct_firstspawn_w) # proportion kelt fecundity
  fecund_s <- fecund_first * prct_firstspawn_s +
    fecund_respawn * (1-prct_firstspawn_s)

  # replace NAs
  fecund_w[is.na(fecund_w)] <- 0
  fecund_s[is.na(fecund_s)] <- 0


  # Weighted egg cap by number of respawners.
  # when all spawners are 1st timers, egg_cap_wt == egg_cap
  # value increases as proportion of kelt increase.
  # at 100% kelt, egg_cap_wt = egg_cap * (fecund_respawn/fecund_first)
  # (egg_cap / fecund_first) = spawnER capacity
  egg_cap_wt_w <- (egg_cap_w / fecund_first) * fecund_w
  egg_cap_wt_s <- (egg_cap_s / fecund_first) * fecund_s

  # percent of successful incoming spawners (S_up*(1-Hr))
  # this assumes time-invariant harvest and prespawn mortality
  surv_ratio_w <- S_up_w*(1-hr)
  surv_ratio_s <- S_up_s*(1-hr)

  # 1st year  ------------------------------------------------------------------
  # spawning happens separately
  eggs_w <- BH_func(
    S = NOR_total_w*surv_ratio_w, p = fecund_w, c = egg_cap_wt_w) # B-H
  eggs_s <- BH_func(
    S = NOR_total_s*surv_ratio_s, p = fecund_s, c = egg_cap_wt_s) # B-H

  # fru colonization happens separately
  pre_fry_w <- eggs_w*egg_fry_surv_w # Eggs --> freshly emerged fry
  pre_fry_s <- eggs_s*egg_fry_surv_s

  pre_fry <- pre_fry_w + pre_fry_s # fry-to-parr stage shared between runs
  percent_fry_w <- safe_divide(pre_fry_w, pre_fry)

  parr <- BH_func(pre_fry, p = parr_surv, c = parr_cap)# summer parr

  parr_distributed <- distrib_fish(
    parr, mat_move_nearest_ms_only, percent_fall_migrants)

  age1 <- BH_func_distrib(parr_distributed, first_winter_surv, first_winter_cap)

  # split runs back up prior to smolting
  age1_w <- age1 * percent_fry_w
  age1_s <- age1 * (1-percent_fry_w)

  # Age1 smolts leave, age1 stay for another year
  # smolts get stored in natal basin, lose affiliation with rearing basin
  diag(mat_out['age1_smolts_winter',,]) <- rowSums(age1_w) * prop_age1_smolts_w
  diag(mat_out['age1_smolts_summer',,]) <- rowSums(age1_s) * prop_age1_smolts_s

  # rearing fish stored as matrix in dimensions 2 and 3 of storage
  # these fish remain affiliated with both their natal and rearing basins
  mat_out['age1_stayers_winter',,] <- age1_w * (1 - prop_age1_smolts_w)
  mat_out['age1_stayers_summer',,] <- age1_s * (1 - prop_age1_smolts_s)

  # 2nd year -------------------------------------------------------------------
  # age 1+, Start with previous year's fish - first fish redistribute to MS,
  # go through summer and winter
  # Spring redistribution of 1+ (0, 20, 50%, depending on basin size)

  # combine remaining fish from both runs
  age1_stayers <- mat_in['age1_stayers_winter', , ]+
    mat_in['age1_stayers_summer', , ]

  # contribution of winter run
  percent_a1stay_w <- safe_divide(
    mat_in['age1_stayers_winter', , ], age1_stayers)

  age1_stayers_dist <- distrib_fish(
    age1_stayers, mat_move_nearest_ms_only, percent_spring_migrants)

  age1_plus <- BH_func_distrib(
    age1_stayers_dist, second_summer_surv, second_summer_cap)

  age2 <- BH_func_distrib(
    age1_plus, second_winter_surv, second_winter_cap) # End of 2nd winter, 2 yo

  # split runs back up prior to smolting
  age2_w <- age2 * percent_a1stay_w
  age2_s <- age2 * (1-percent_a1stay_w)

  # Age2 smolts leave, age2 stay for another year
  diag(mat_out['age2_smolts_winter',,]) <- rowSums(age2_w) * prop_age2_smolts_w
  diag(mat_out['age2_smolts_summer',,]) <- rowSums(age2_s) * prop_age2_smolts_s

  # rearing fish stored as matrix
  mat_out['age2_stayers_winter', , ] <- age2_w * (1 - prop_age2_smolts_w)
  mat_out['age2_stayers_summer', , ] <- age2_s * (1 - prop_age2_smolts_s)


  # 3rd year,
  # age 2+ stayers from previous year rear for 1-2 more years
  age2_stayers <- mat_in['age2_stayers_winter', , ]+
    mat_in['age2_stayers_summer', , ]
  percent_a2stay_w <- safe_divide(
    mat_in['age2_stayers_winter', , ], age2_stayers)

  age2_plus <- BH_func_distrib(
    age2_stayers, p = second_summer_surv, c = second_summer_cap)
  age3 <- BH_func_distrib(
    age2_plus, p = second_winter_surv, c = second_winter_cap)

  # split runs back up prior to smolting
  # all remaining freshwater fish smolt at this time
  diag(mat_out['age3_smolts_winter',,]) <- rowSums(age3 * percent_a2stay_w)
  diag(mat_out['age3_smolts_summer',,]) <- rowSums(age3 * (1-percent_a2stay_w))


  # Ocean stages - before any spawning

  # first year ocean fish after downstream migration and bay survival
  # walk over year1 saltwater and smolt names in parallel, fill in ocean stage
  # based on bay survival of smolts

  # ocean entry for winter run fw ages 1-3
  # this evaluates in similar time whether or not each ls is run sequentially
  mat_out[sprintf('age%s_1_ocean_winter', 1:3),,] <-
    mat_in[sprintf('age%s_smolts_winter', 1:3),,] *
    bay_surv_w * so

  # and for summer run
  mat_out[sprintf('age%s_1_ocean_summer', 1:3),,] <-
    mat_in[sprintf('age%s_smolts_summer', 1:3),,] *
    bay_surv_s * so

  # second ocean year - with spawners removed

  runs <- c("winter", "summer")

  # fw 1 fish
  # Here we reduce the maturation and survival multipliers down to length 2
  # vectors. Each "slice" of the array is multiplied by either the first
  # (winter) or second (summer) number so that we can multiply our 3D array
  # by something of the correct dimensions
  # the other options would be to expand the multipliers into an array for 1:1
  # element-wised multiplication, or to assign each run separately.
  # With this method, maturation rates cannot vary by subbasin
  mat_out[sprintf('age1_2_ocean_%s', runs),,] <-
    mat_in[sprintf('age1_1_ocean_%s', runs),,] *
    (1 - c(b1_1_w[1], b1_1_s[1])) * so[1]

  mat_out[sprintf('age1_3_ocean_%s', runs),,] <-
    mat_in[sprintf('age1_2_ocean_%s', runs),,] *
    (1 - c(b1_2_w[1], b1_2_s[1])) * so[1]

  # fw 2 fish
  mat_out[sprintf('age2_2_ocean_%s', runs),,] <-
    mat_in[sprintf('age2_1_ocean_%s', runs),,] *
    (1 - c(b2_1_w[1], b2_1_s[1])) * so[1]

  mat_out[sprintf('age2_3_ocean_%s', runs),,] <-
    mat_in[sprintf('age2_2_ocean_%s', runs),,] *
    (1 - c(b2_2_w[1], b2_2_s[1])) * so[1]

  # fw 3 fish
  mat_out[sprintf('age3_2_ocean_%s', runs),,] <-
    mat_in[sprintf('age3_1_ocean_%s', runs),,] *
    (1 - c(b3_1_w[1], b3_1_s[1])) * so[1]

  # Age structure of mixed adult returns
  mat_out[sprintf('age1_1_spawn_%s', runs),,] <-
    mat_in[sprintf('age1_1_ocean_%s', runs),,] *
    c(b1_1_w[1], b1_1_s[1])

  mat_out[sprintf('age1_2_spawn_%s', runs),,] <-
    mat_in[sprintf('age1_2_ocean_%s', runs),,]*
    c(b1_2_w[1], b1_2_s[1])

  mat_out[sprintf('age1_3_spawn_%s', runs),,] <-
    mat_in[sprintf('age1_3_ocean_%s', runs),,] # full cohort returns


  mat_out[sprintf('age2_1_spawn_%s', runs),,] <-
    mat_in[sprintf('age2_1_ocean_%s', runs),,] *
    c(b2_1_w[1], b2_1_s[1])

  mat_out[sprintf('age2_2_spawn_%s', runs),,] <-
    mat_in[sprintf('age2_2_ocean_%s', runs),,] *
    c(b2_2_w[1], b2_2_s[1])

  mat_out[sprintf('age2_3_spawn_%s', runs),,] <-
    mat_in[sprintf('age2_3_ocean_%s', runs),,] # full cohort returns

  mat_out[sprintf('age3_1_spawn_%s', runs),,] <-
    mat_in[sprintf('age3_1_ocean_%s', runs),,] *
    c(b3_1_w[1], b3_1_s[1])

  mat_out[sprintf('age3_2_spawn_%s', runs),,] <-
    mat_in[sprintf('age3_2_ocean_%s', runs),,] # full cohort returns


  # NOTE: How do we turn "off" harvest when we have kelts that have been subject
  # to harvest twice, once as first time returning spawners and again as
  # returning respawners?
  # calculate kelt from previous year's run
  mat_out[sprintf('kelt_%s', runs),, ] <-
    mat_in[sprintf('total_run_%s', runs),, ] * kelt_rate

  # calculate total first time spawners from mixed-age returners
  diag(mat_out['new_spawners_winter',, ]) <- apply( # sum by subbasin
    mat_out[grepl("spawn_winter", rownames(mat_out)),,],2,sum)

  diag(mat_out['new_spawners_summer',, ]) <- apply( # sum by subbasin
    mat_out[grepl("spawn_summer", rownames(mat_out)),,],2,sum)


  # total run is first time spawners + repeat spawners calculated above
  mat_out['total_run_winter',, ] <- mat_out['new_spawners_winter',, ]+
    mat_out['kelt_winter',, ]

  mat_out['total_run_summer',, ] <- mat_out['new_spawners_summer',, ]+
    mat_out['kelt_summer',, ]

  # apply prespawn mortality and harvest, if implemented
  mat_out['spawners_winter',, ] <-  mat_out['total_run_winter',, ] *
    S_up_w * # prespawn mortality
    (1 - hr) # harvest survival

  mat_out['spawners_summer',, ] <-  mat_out['total_run_summer',, ] *
    S_up_s * # prespawn mortality
    (1 - hr) # harvest survival

  diag(mat_out["spawners",, ]) <- apply( # sum spawners from both runs
    mat_out[sprintf('spawners_%s', runs),, ], 2, sum)

  # Other stored stages

  diag(mat_out['eggs_winter',, ]) <- eggs_w
  diag(mat_out['eggs_summer',, ]) <- eggs_s

  diag(mat_out['pre_fry_winter',, ]) <- pre_fry_w
  diag(mat_out['pre_fry_summer',, ]) <- pre_fry_s

  diag(mat_out['parr',, ]) <- parr

  # sum of smolts
  diag(mat_out['smolts_winter',, ]) <- apply(
    mat_out[grepl("smolts_winter", rownames(mat_out)),,], 2, sum)
  diag(mat_out['smolts_summer',, ]) <- apply(
    mat_out[grepl("smolts_summer", rownames(mat_out)),,], 2, sum)

  return(mat_out)

} #end steelhead
