# a set of utilities to model suitable beaver habitat

#' Convert factor label into numeric
#' @param x a factor with number labels
as.numeric.factor <- function(x) {as.numeric(levels(x))[x]}

# BIP model described in Dittbrenner et al. 2018.
# Slighlty modified to class cumulative scores of 8 as 0 BIP

#' Convert numeric vector into BIP subscore
#' @param x vector to be classed
#' @param param one of "slope", "bfw", "valley" and "cuml"
score_var <- function(x, param){
  # breaks for cut function
  slope_breaks <- c(0, 1, 2, 4, 6, 10, Inf)
  slope_score <- c(4, 3, 2, 1, 0.5, 0)

  bfw_breaks <- c(0, 7, 10, 18, 24, Inf)
  bfw_score <- c(4, 3, 2, 1, 0)

  valley_breaks <- c(0, 20, 30, Inf)
  valley_score <- c(0, 2, 4)

  cuml_breaks <- c(0, 8, 10, 11, Inf)
  cuml_score <- c(0, 1, 2, 3)

  cut(
    x,
    breaks = eval(parse(text = sprintf("%s_breaks", param))),
    labels = eval(parse(text = sprintf("%s_score", param))),
    include.lowest = T,
    right = F
  ) %>%
    as.numeric.factor()
}

#' Add BIP scores to a tibble
#' @param x Tibble or data frame of habitats
#' @param slope stream gradient
#' @param bfw column containing stream bankful width in meters
#' @param val_width column containing valley width in meters
#' @param keep_subscores retain subscores?
calc_BIP <- function(x, slope, bfw, val_width, keep_subscores = FALSE) {
  BIP <- x %>%
    mutate.(
      perc_grad = {{slope}}*100,
      slope_score = score_var(perc_grad, "slope"),
      bfw_score = score_var({{bfw}}, "bfw"),
      valley_score = score_var({{val_width}}, "valley"),
      total_score = slope_score + bfw_score + valley_score,
      beaver_score = score_var(total_score, "cuml"),
      beav_hab = total_score > 8 )
  if(!keep_subscores){
    BIP <- BIP %>%
      select.(-c(
        perc_grad,
        slope_score,
        bfw_score,
        valley_score,
        total_score,
        beaver_score
        ))
  }
  return(BIP)
}

# Beaver habitat stream power model described in Pollock et al. 2004.

#' Calculate stream power
#' @param slope stream gradient
#' @param DA drainage area in km^2
calc_sp <- function(slope, DA) 7.15 * 10^3*DA*slope

#' Add stream power and beaver suitability rating to table
#' @param x Tibble or data frame of habitats
#' @param slope column containing stream gradient
#' @param DA column containing drainage area in km^2
#' @param keep_subscores retain subscores?
calc_pollock <- function(x, slope, DA, keep_subscores = FALSE){
  beav <- x %>%
    mutate.(
      sp = calc_sp({{slope}}, {{DA}}),
      beav_hab = ({{slope}} <= 0.04 & {{DA}} >= 0.1 & sp <= 2000)
    )
  if(!keep_subscores){
    beav <- beav %>%
      select.(-sp)
  }
  return(beav)
}
