
# HARP Model for the Stillaguamish River Basin

The `harp` package is adapted from the `crp` package and `asrp` model,
both developed for use in the Chehalis River basin. The `asrp` model was
created to assess diagnostic and restoration scenarios presented in
contract reports (Beechie et al. 2021) as well as several manuscripts
(Habitat analysis, LCM, Flow and Temperature papers). `crp` was built on
the foundation of the `asrp` model but was converted into a package, and
adapted to allow for stochastic analysis. The `harp` package is being
developed for use in the Stillaguamish basin.

## Installation

To install the latest version of the model from GitHub, run

    remotes::install_github('noaa-lcm/harp_stillaguamish.git')

To install the model from a local directory, run

    devtools::install()

## Intended usage

At a high level there are three functions which need to be used to
generate `harp` model results:

-   `hab_run()`
-   `lcm_assign()`
-   `lcm_run()`

These functions can be chained using the magrittr pipe operator `%>%`

``` r

# deterministic
hab_run('coho', 'Current', 'curr') %>%  
  lcm_assign() %>%  
  lcm_run() 

# stochastic
hab_run_stochastic('coho', 'Current', 'curr', runs = 1:20, years = 1:100) %>%
  lcm_assign() %>%
  lcm_run()
```

## Examples

### Deterministic

Create all habitat scenarios for a given species, scenario, and era

``` r
habitat_results <- hab_run('coho', 'Current', 'curr')
```

Run the LCM for a specific species and scenario. If you want to see the
equilibrium values, subset the results to only the spawners in year 100.

``` r

lcm_inputs <- lcm_assign(habitat_results)

lcm_run(lcm_inputs) %>% 
  dplyr::filter(year == 100)
```

Summer- and fall-run Chinook salmon are grouped together under
`fall_chinook` within the model code.

``` r
hab_run('fall_chinook', 'Floodplain', 'curr') %>%
  lcm_assign() %>%
  lcm_run()
```

A special steelhead function, `lcm_assign_steelhead`, is used to run the
steelhead lifecycle model. This function takes as inputs the habitat
model outputs for winter- and summer-run steelhead

``` r
w_steelhead_hab <- hab_run('winter_steelhead', 'Barriers', 'curr')
s_steelhead_hab <- hab_run('summer_steelhead', 'Barriers', 'curr')

lcm_assign_steelhead(w_steelhead_hab, s_steelhead_hab) %>%
  lcm_run()
```

By default, `lcm_run` returns only numbers of returning spawners for
each subpopulation. However, additional lifestages can be specified by
name. `lcm_run('all')` returns all lifestages.

``` r
hab_run('fall_chinook', 'LR', 'curr') %>%
  lcm_assign() %>%
  lcm_run(c('parr_migrants', 'spawners'))
```

### Stochastic

Data from the MARSS simulations can be used to create stochastic runs of
the lifecycle model. Here we simulate a sequence of 100 years using
`hab_run_stochastic`. `runs` specifies which sequence or sequences of
years are used.

``` r
chk_stoch_hab <- hab_run_stochastic(
  'fall_chinook', 
  'Current', 
  'curr',
  runs = 1,
  years = 1:100)

lcm_assign(chk_stoch_hab) %>%
  lcm_run()
```

The HARP package includes specialized `print` and `plot` methods for
lifecycle model outputs, which can be useful for visualizing stochastic
model outputs. The print method displays the median basinwide number of
spawners.

``` r
coh_stoch_hab <- hab_run_stochastic(
  'coho', 
  'Current', 
  'curr',
  runs = 1:10,
  years = 1:50)

coho_stoch_lcm <- lcm_assign(coh_stoch_hab) %>%
  lcm_run()

print(coho_stoch_lcm)
#> Summary:
#>    pop  era scenario spawners
#> 1 coho curr  Current   13,364
#> 
#> Preview:
#> # A tibble: 17,000 × 8
#>    pop   scenario era     run  year lifestage subbasin                     n
#>    <chr> <chr>    <chr> <dbl> <dbl> <fct>     <fct>                    <dbl>
#>  1 coho  Current  curr      1     1 spawners  mainstem_Stillaguamish 0.00998
#>  2 coho  Current  curr      1     2 spawners  mainstem_Stillaguamish 0.0160 
#>  3 coho  Current  curr      1     3 spawners  mainstem_Stillaguamish 0.00749
#>  4 coho  Current  curr      1     4 spawners  mainstem_Stillaguamish 0.00467
#>  5 coho  Current  curr      1     5 spawners  mainstem_Stillaguamish 0.00648
#>  6 coho  Current  curr      1     6 spawners  mainstem_Stillaguamish 0.00865
#>  7 coho  Current  curr      1     7 spawners  mainstem_Stillaguamish 0.00321
#>  8 coho  Current  curr      1     8 spawners  mainstem_Stillaguamish 0.00202
#>  9 coho  Current  curr      1     9 spawners  mainstem_Stillaguamish 0.00318
#> 10 coho  Current  curr      1    10 spawners  mainstem_Stillaguamish 0.00349
#> # … with 16,990 more rows
```

By default, the lcm plot function produces a ggplot that will show
individual stochastic time series in grey, median annual values in
black, and the overall median value as a blue dotted line.

``` r
plot(coho_stoch_lcm)
```

![](man/figures/README-show%20plot-1.png)<!-- -->
