# Read in and prepare datasets from CSVs

library(tidyverse)

# Spatial model outputs --------------------------------------------------------
# Files read from most recent Chehalis spatial model outputs folder iteration
# If spatial model is re-run, copy spatial model output files to
# "data-raw/data_hab/hab_input_data/spatial_model_outputs" and re-run
# "data-raw/create-data.R"

# Flowline ----

# Streams greater than 20m BFW that should be considered small streams anyway
SmStream_reclass <- scan(
  "data-raw/data_hab/STL_SmStream_manual_class.txt",
  quiet = TRUE)

flowline_raw <- list.files(
  "data-raw/data_hab/spatial_model_outputs",
  pattern = "flowline",
  full.names = T) %>%
  read.csv() %>%
  filter(Habitat != "Tidal") %>%
  mutate(
    Habitat = case_when( # reclass stream habitat
      noaaid %in% SmStream_reclass ~ "SmStream",
      BF_width >= 20 ~ "LgStream",
      BF_width < 20 ~ "SmStream"
    ))

if(length(unique(flowline_raw$noaaid)) != nrow(flowline_raw)){
  stop("Each flowline row must have unique noaaid.")
}

# Large river spawning riffles.  Hand digitized ----
riff <- list.files(
  path = "data-raw/data_hab/spatial_model_outputs",
  pattern = "Riffles",
  full.names = T) %>%
  read.csv()

# Large river habitat data ----
# update to select used fields rather than unused

# conversion table used to simplify LR habitat fields
lookup_table <- tribble(
  ~HabUnit,         ~Habitat,       ~Period,
  "Bank",           "Bank",         "Both",
  "Bank Ar",        "Bank",         "Hist", # All banks historically non-armored
  "Bank Ar",        "HM_Bank",      "Curr",
  "Boulder Riffle", "Bar_boulder",  "Both",
  "Bar",            "Bar_gravel",   "Both", # sand bars are rare
  "Bar Ar",         "Bar_gravel",   "Both"  # currently not handling bar armor
)

EdgeHabs_raw <- list.files(
  path = "data-raw/data_hab/spatial_model_outputs",
  pattern = "EdgeHabs",
  full.names = T) %>%
  read.csv() %>%
  left_join(lookup_table, by = "HabUnit") %>%
  select(
    Habitat,
    Length_m,
    noaaid,
    Period
  )

# Backwater Habitat data ----
# update to select used fields rather than unused
Backwater_raw = list.files(
  path = "data-raw/data_hab/spatial_model_outputs",
  pattern = "Backwater",
  full.names = T) %>%
  read.csv()  %>%
  mutate(Period = "Both") %>%
  select(Area_ha, Period, noaaid)

# Floodplain habitat data ----
Floodplain_raw <- list.files(
  path = "data-raw/data_hab/spatial_model_outputs",
  pattern = "Floodplain",
  full.names = T) %>%
  read.csv() %>%
  select(
    HabUnit,
    Area_ha,
    Period,
    Hab_cond,
    bfw_conn, # within 5m of 1/2 BFW of flowline
    in_fp, # in PSP floodplain
    noaaid
  ) %>%
  mutate(
    bfw_conn = replace_na(bfw_conn, FALSE),
    Habitat = case_when(
      HabUnit %in% c("Pond", "Slough", "Lake") & Area_ha < .05 ~ "FP_Pond_sm",
      HabUnit %in% c("Pond", "Slough", "Lake") & Area_ha < 5 ~ "FP_Pond_lg",
      HabUnit %in% c("Pond", "Slough", "Lake") & Area_ha >= 5 ~ "Lake",
      HabUnit == "Side channel" ~ "Side_Channel",
      HabUnit == "Marsh" ~ "Marsh"
    )
  )

SC_curr_raw <- list.files(
  path = "data-raw/data_hab/",
  pattern = "est_SC",
  full.names = T
) %>%
  read.csv() %>%
  select(noaa_subba, est_sc_km)

# Other inputs -----------------------------------------------------------------

## code to prepare `subbasin_scenario_names` dataset goes here
# Generated from subbasin shapefile
subbasin_names <- list.files(
  path = "data-raw/data_hab/spatial_model_outputs",
  pattern = "Subbasin",
  full.names = T) %>%
  read.csv() %>%
  mutate(subb_area = Shape_Area/1000^2) %>%
  select(
    noaa_subba,
    subbasin_num,
    is_ms, nearest_ms, downstr_ms,
    subb_area,
    subb_group) %>%
  arrange(as.numeric(subbasin_num))

# custom scenarios
custom_scen <- read.csv('data-raw/data_hab/STL_custom_scenarios.csv') %>%
  mutate(scenario = str_replace_all(scenario, '\\.', '_'))

# CSV generated by hand, built-in scenario descriptions
scenario_desc <- read.csv('data-raw/data_hab/scenario_description.csv') %>%
  mutate(scenario = str_replace_all(scenario, '\\.', '_'))


# Large river length multipliers ----
lr_length_raw <- read.csv("data-raw/data_hab/stl_LR_Length.csv") %>%
  left_join(subbasin_names, by = "noaa_subba")

# Pool ratio by slope and landcover
pool_ratios <- read.csv("data-raw/data_hab/stl_pool_ratios.csv")

# EDT summer and winter wetted widths.
# Contains current, historical, 2040 and 2080 widths ----
# tag: ReplaceData
# edt_width_raw <- read.csv("data-raw/data_hab/edt_width.csv") %>%
#   mutate(Reach_low = tolower(Reach)) %>%
#   rename(era = year) %>%
#   mutate(era = case_when(era == 1900 ~ 'hist',
#                          era == 2019 ~ 'curr',
#                          era == 2040 ~ 'mid',
#                   era == 2080 ~ 'late',
#                          TRUE ~ 'error'))

# Future impervious area ----
# tag: ReplaceData
# conv_perc <- function(x) as.numeric(gsub("%", "", x)) / 100 #percent to decimal
#
# fut_imperv <- read.csv(
#   'data-raw/data_hab/future_impervious.csv') %>%
#   rename(mid = Mid.century.Added.Impervious.Area, # switch to era codes
#          late = Late.century.Added.Impervious.Area) %>%
#   select(GSU, mid, late) %>%
#   mutate(mid = conv_perc(mid),
#          late = conv_perc(late)) %>%
#   gather(era, future_imperv, mid, late) %>%
#   group_by(era, GSU) %>%
#   summarize(future_imperv = sum(future_imperv, na.rm = T))

