
# First run data-raw/hab_read_datasets.R and data-raw/data_hab/flowline_prep.R

# Floodplain reconnection ------------------------------------------------------
# streams that are connected to their floodplain will become cooler, based on
# width. Steep and confined streams are considered unable to be connected to
# any floodplain
cool_floodplain <- function(width){
  cut(
    width,
    breaks = c(-Inf, 10, 20, 30, Inf), # widths at which to divide streams
    labels = c(-0.29, -0.72, -1, -1.43), # cooling factor.
    # A 25m stream is cooled by 1 degree. A 9 m stream is cooled by 0.29 degrees
    right = FALSE,
  ) %>% as.character() %>% as.numeric() # return numeric result
}

# Convert August to June temperatures-------------------------------------------
# temp_conversion_scalar is based on linear regression between the two
# temperature metrics
t7DADM_to_Jn121_func <- function(dadm7) {
  temp_conversion_scalar * dadm7
}

# Subset flowline layer to columns necessary for temperature calculations ------
flowline_temps <- flowline_raw %>%
  select(
    noaaid,
    Habitat,
    BF_width,
    temp_7DADM,
    temp_Jn121,
    angle_chg,
    slope,
    left_ht,
    right_ht
    )

# Create temperature_df --------------------------------------------------------
# former mdm is temp_Jn121, former mwmt is temp_7DADM
temperature_df <- expand_grid( # all combinations of flowline, era
  flowline_temps,
  era = eras_all) %>%
  left_join(scenarios_df, by = "noaaid") %>%
  mutate(
    # temperature differences from floodplain restoration
    temp_diff_fp = case_when(
      # no restoration in confined reaches or reaches above 3% grade
      slope > 0.03 | confined ~ 0, # confined is a logical (TRUE/FALSE)
      # otherwise, cool the floodplain based on width and rest intensity
      TRUE ~ cool_floodplain(BF_width) * rest_perc * fp_intensity_scalar),

    # these next fields would vary by era in future scenarios
    # baseline temps are current temp for hist and curr eras, but would be
    # future modeled temps for mid and late eras
    temp_baseline_Jn121 = temp_Jn121,
    temp_baseline_7DADM = temp_7DADM,

    # canopy-change based temperatures are just the historical - current
    # temperature difference for hist era and curr shade scenarios.

    # but for future scenarios, we'd use modeled temperature difference
    # between future and present scenario
    temp_diff_hist = -0.035*angle_chg, #Seixas et al. 2018, historic - current
    temp_diff_canopy = case_when(
      era == 'hist' ~ temp_diff_hist,
      era == 'curr' ~ temp_diff_hist * rest_perc* temp_intensity_scalar),
    # 'temp_diff_canopy_noaction' would vary by era.
    # we want to scale temperature differences to disallow growth of short
    # (<15 m) riparian veg outside of Riparian restoration scenarios
    # Outside of riparian rest scenarios, we want "veg" <15m NOT to grow

    # final temperatures are the sum of baseline temperatures and any riparian
    # or floodplain connectivity-based changes
    temp_Jn121 = temp_baseline_Jn121 +
      t7DADM_to_Jn121_func(temp_diff_canopy + temp_diff_fp),
    temp_7DADM = temp_baseline_7DADM + temp_diff_canopy + temp_diff_fp,
    temp_prespawn = temp_7DADM # not useds
    ) %>%
  select(noaaid, era, scenario, temp_Jn121, temp_7DADM, temp_prespawn)
