# Stillaguamish version of Jeff Jorgensen's Chehalis Time Series RMD
# To simulate time series while preserving correlations and covariances between them, we can use MARSS models
# The following represent Stillaguamish peak flow, low flow, and air temperature data time series to forward simulate

require(MARSS)
library(tidyverse)

#Data used:
#USGS NF near Arlington Stillaguamish gage station for peak and low flow data (1928-2021)
#NOAA NCDC Monroe air station maximum air temperature
#NorWest lower mainstem Stillaguamish stream temperature data (2001- 2012)

# The peak flow data will be summarized for the incubation period for each population
# Stillaguamish incubation periods from Tim Beechie's memo

incubation_range <- list(
  c(pop = 'coho',           start = "1020", end = "0515"),
  c(pop = 'summer_chinook', start = "0801", end = "0415"),
  c(pop = 'fall_chinook',   start = "0901", end = "0510"),
  c(pop = 'winter_steelhead', start = "0215", end = "0910"),
  c(pop= 'summer_steelhead', start = "0101", end = "0910"))

# Daily discharge at NF Stillaguamish near Arlington
# helpful webpages: http://usgs-r.github.io/EGRET/
# https://usgs-r.github.io/dataRetrieval/
require(EGRET)
site_number_stilly <- 12167000 #NF Stillaguamish gage
start_date <- "1929-01-01" # Get earliest date
end_date <- "2020-12-31"
daily_stilly <- readNWISDaily(site_number_stilly,"00060",start_date,end_date)

# Find the annual maximum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
stilly_q_peak <- map_dfr(incubation_range, function(r){
  daily_stilly %>%
    mutate(monthday = format(Date, "%m%d"),
           pop = r['pop']) %>%
    filter(monthday <= r['start'] | monthday >= r['end']) %>%
    group_by(pop, waterYear) %>%
    summarize(Q_max = max(Q) * 35.31467)
})
stilly_q <- pivot_wider(stilly_q_peak,
                        names_from = pop,
                        names_prefix = 'peak_q_',
                        values_from = Q_max)
# Find the annual minimum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
# Calculating 7DADMin for low flow
stilly_low_q7 <- tapply(daily_stilly$Q7[daily_stilly$waterYear>1927],
                        daily_stilly$waterYear[daily_stilly$waterYear>1927],
                        function(x, n = 7) {
                          min(as.vector(stats::filter(x, rep(1/n, n),sides=2)),
                              na.rm = TRUE)
                        })
stilly_low_q7 <- data.frame(waterYear = names(stilly_low_q7),
                            low_q7 = stilly_low_q7 * 35.31467)

stilly_q <- merge(stilly_q, stilly_low_q7)

# Stream temperature data from NorWest lower Stillaguamish
# Used to create relationship between max air temp and max water temp
stilly_stream_temp <- read.csv("Stilly_lower_stream_temp.csv")%>%
  mutate(start_date = as.Date(start_date),
         end_date = as.Date(end_date))


# Air temperature data
# USC00455525 is the ID for the Snohomish Monroe NOAA NCDC station
require(rnoaa)
require(purrr)

t_max <- map(c("USC00455525"), ghcnd_search,
             var = 'TMAX',
             date_min = '1920-01-01',
             date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y")))%>% #year only format for year_1 column
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)



# We fit a relationship between NorWest  Stillaguamish 7DADM water temperature and Snohomish Monroe TMAX air temperature
# This relationship is used later to convert the MARSS simulations
# of air TMAX to water 7DADM in the life cycle model

# Subset air temperature time period to match stream temperature time series
t_max_regression <- map(c("USC00455525"), ghcnd_search,
             var = 'TMAX',
             date_min = '1920-01-01',
             date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y"))) #year only format for year_1 column

date_subset <- function(start_date, end_date){
  filter(t_max_regression, date>=start_date, date<=end_date)
}

t_max_subset <- map2_dfr(stilly_stream_temp$start_date, stilly_stream_temp$end_date, date_subset)%>%
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)#NOAA data is downloaded as max temperature in tenths of degrees C

stream_air_temp <- merge(x=stilly_stream_temp,
                        y=t_max_subset, all.x = TRUE)
# Regression
lm_7dadm_tmax <- lm(T7dadm ~ tmaxC, stream_air_temp)
summary(lm_7dadm_tmax)

# Prepare data, fit MARSS model
# Before bringing data into a MARSS model, transform data that is not normally distributed
# Here, we log transform the low flow and tmax data and square root transform the summer steelhead peak flow data
# Then center and scale (z-score) the data
stilly <- stilly_q
stilly[,c(7)]  <- log(stilly[,c(7)]) #log low flow only
stilly[,c(6)]  <- sqrt(stilly[,c(6)]) #sqrt summer steelhead peak flow only
stilly[,c(2)] <- sqrt(stilly[,c(2)]) #square root transform coho/chinook peak flow
stilly[,c(5)] <- sqrt(stilly[,c(5)]) #square root transform winter steelhead peak flow

# Coho and chinook have a 1:1 match, so drop chinook columns
stilly <- scale(stilly[,c(2,5,6,7)])

# Column means & standard deviations needed later for back-transforming
stilly_mean <- attributes(stilly)$'scaled:center'
stilly_std <- attributes(stilly)$'scaled:scale'
stilly <- t(stilly)

# Transform and z-score TMAX air temperature
air_t <- t_max
air_t[,c(2)]  <- log(air_t[,c(2)])
air_t <- scale(air_t[, 2])

# Column means & stdevs needed later for back-transforming
air_t_mean <- attributes(air_t)$'scaled:center'
air_t_std <- attributes(air_t)$'scaled:scale'
air_t <- t(air_t)

# Combining data
dat_full <- rbind(stilly, air_t)
rownames(dat_full)[5] <- 'air_t'
colnames(dat_full) <- 1929:2021
numYears<-ncol(dat_full)

# Fitting MARSS model
dat <- dat_full
Vars <- row.names(dat)
numVars <- length(Vars)
years <- 1929:2021

# MARSS fit
mod.list.noint <- list(
  B="diagonal and unequal", #why is this mean reverting instead of a random walk (diagonal and equal)
  U="zero",
  Q="unconstrained", #most parameterized option for estimated error correlatrion structure
  Z="identity", #grouping hypothesis matrix
  A="zero", #not sure what this is -- growth term?
  R="zero", #error structure -- is this because we have different response variables?
  x0="zero",
  tinitx=0 )


ts.ch.1 <- MARSS(dat,
                 model = mod.list.noint,
                 fun.kf ="MARSSkfss",
                 control=list(maxit=5000,
                              conv.test.slope.tol=0.001,
                              trace=1,
                              safe=TRUE), silent = TRUE)

# Simulate data
# We take the fitted MARSS model and use it to forward simulate data
# We can repeat this process to generate time series for life cycle model simulations
simYears<-150 #150 years
numSims<-100 #100 simulations
mySims <- MARSSsimulate(ts.ch.1, tSteps = simYears, nsim = numSims)

# Back-transform data
#r reverse-transform}
#https://stackoverflow.com/questions/46397553/r-transform-scaled-and-centered-data-to-original-values
# value * 'scaled:scale' + 'scaled:center'
# = value*std + mean
# lowQ, peakQ, airTMAX
# back-transform lowQ, peakQ
test <- apply(mySims$sim.data, MARGIN=c(2,3), function(x) x * t(c(stilly_std, air_t_std)) +
                t(c(stilly_mean, air_t_mean)))
test2 <- test
#test2[,,1] lets you see the entire back transformed simulated data for 1 simulation

#one more step:
#inverse log transform of low flow and max air temp
#inverse square root transform (i.e., square) peak flow summer steelhead

for(i in 1:dim(test2)[3]){ #why the [3]?
  test2[3,,i] <- (test2[3,,i]^2)
  test2[1,,i] <- test2[1,,i]^2
  test2[2,,i] <- test2[2,,i]^2
  test2[4,,i] <- exp(test2[4,,i])
  test2[5,,i] <- exp(test2[5,,i])
}

# Add data to `harp` package

#Store the unscaled version of the three MARSS forecasts. This makes them available for other functions to call.
dat_marss <- test2

dimnames(dat_marss) <- list(#var = rownames(mySims$MLEobj$model$data),
  var = c('q_peak', 'q_peak_win_sthd', 'q_peak_sum_sthd', 'q_low', 't_peak'),
  year = 1:simYears,
  run = 1:numSims)


# Convert array to dataframe
df_marss <- as.data.frame.table(dat_marss) %>%
  rename(value = Freq)

