# calibrate SAR and age structure for steelhead
devtools::load_all(export_all = F)
library(tidyverse)
library(glue)

chinook_base <- hab_run("fall_chinook", "Current", "curr") %>% lcm_assign()

# how many spawners do we have with default parameters?
chinook_base %>%
  lcm_run() %>%
  filter(year == 100) %>%
  summarize(n = sum(n))

# calibration targets
fry_ratio_target <- 26.8/73.2 # ratio from smolt trap
fry_vector <- c(0.268, 0.732)
SAR_target <- 0.012  # target SAR
subyearling_target <- c(0.99, 0.01) # yearlings avg 1%
nRunGlobal <- 2000
nRunLocal <- 2000


# use fry ratio in RMSE
calc_everything <- function(x){

  all_lcm <- chinook_base %>%
    mutate(
      subyr_ratio = x[1],
      ps_parr_surv = x[2],
      fry_migr_prod = x[3]
    )  %>%
    lcm_run("all") %>%
    filter(year == 100)

  fry_ratio <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = fry_ratio_target) %>%
    mutate(
      fry_ratio = fry_migrants/parr_migrants,
      fry_diff = ((fry_ratio - target)/100)^2) %>%
    pull(fry_diff)

  parr_SAR <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants", "total_run_subyr")) %>%
    mutate(
      lifestage = ifelse(grepl("migrants", lifestage), "smolt", "adult")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = c(SAR_target)) %>%
    mutate(
      SAR = adult/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  subyr <- all_lcm %>%
    filter(
      grepl("total_run_", lifestage)) %>%
    mutate(
      fw_age = stringr::str_extract(lifestage, "(?<=total_run_)\\w+"),
    ) %>%
    group_by(fw_age) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    ungroup() %>%
    mutate(perc = n/sum(n)) %>%
    filter(fw_age == "yr") %>% # match yearlings (matching both is redundant)
    cbind(target = subyearling_target[2])%>%
    mutate(sq_diff = ((perc-target)/10)^2) %>%
    pull(sq_diff)

  sqrt(sum(fry_ratio, parr_SAR*2, subyr)/4) #weight SARs 2x that of the ages
}

# Run global optimization ------------------------------------------------------
start_time <- Sys.time() # time it
nlopt1 <- nloptr::nloptr( # run nonlinear optimizer
  x0 = c(0.98,
         0.056,
         0.10),
  eval_f = calc_everything,
  lb = rep(0,3),
  ub = rep(1,3),
  opts = list(
    algorithm ="NLOPT_GN_DIRECT_L",
    print_level = 1,
    maxeval = nRunGlobal # 2000 iterations
  )
)
results1 <- nlopt1$solution # store results of global optimization
elapsed_time1 <- (Sys.time() - start_time)

# Run global optimization ------------------------------------------------------
start_time2 <- Sys.time()
nlopt2 <- nloptr::nloptr( # run nonlinear optimizer
  x0 = results1, # start with global results
  eval_f = calc_everything,
  lb = rep(0,3),
  ub = rep(1,3),
  opts = list(
    algorithm ="NLOPT_LN_COBYLA",
    print_level = 1,
    maxeval = nRunLocal # 2000 iterations
  )
)
results2 <- nlopt2$solution
elapsed_time2 <- (Sys.time() - start_time2)
elapsed_total <- as.numeric(elapsed_time1+elapsed_time2, units = "mins")

# extract model results -------------------------------------------------------

# get Neq with calibration
calibrated_results <- chinook_base %>%
  mutate(
    subyr_ratio = results2[1],
    ps_parr_surv = results2[2],
    fry_migr_prod = results2[3])  %>%
  lcm_run("all") %>%
  filter(year == 100)

spawners_model <- calibrated_results %>%
  filter(lifestage == "spawners") %>%
  summarize(n = sum(n))

spawners_harvest <- chinook_base %>%
  mutate(
    subyr_ratio = results2[1],
    ps_parr_surv = results2[2],
    fry_migr_prod = results2[3],
    hr = 0.3)  %>%
  lcm_run("all") %>%
  filter(year == 100)%>%
  filter(lifestage == "spawners") %>%
  pull(n) %>%
  sum()

spawners_model <- calibrated_results %>%
  filter(lifestage == "spawners") %>%
  summarize(n = sum(n))

SAR_model <- calibrated_results %>%
  filter(
    lifestage %in% c("parr_migrants", "fry_migrants", "total_run_subyr")) %>%
  mutate(
    lifestage = ifelse(grepl("migrants", lifestage), "smolt", "adult")) %>%
  group_by(lifestage) %>%
  summarize(n = sum(n), .groups = "keep") %>%
  pivot_wider(names_from = lifestage, values_from = n) %>%
  cbind(target = c(SAR_target)) %>%
  mutate(SAR = adult/smolt) %>%
  pull(SAR)

yearlings <- calibrated_results %>%
  filter(
    grepl("total_run_", lifestage)) %>%
  mutate(
    fw_age = stringr::str_extract(lifestage, "(?<=total_run_)\\w+"),
  ) %>%
  group_by(fw_age) %>%
  summarize(n = sum(n), .groups = "keep") %>%
  ungroup() %>%
  mutate(perc = n/sum(n)) %>%
  filter(fw_age == "yr") %>%
  pull(perc)

outmigrants <- calibrated_results %>%
  filter(lifestage %in% c("parr_migrants", "fry_migrants")) %>%
  group_by(lifestage) %>%
  summarize(n = sum(n), .groups = "keep") %>%
  ungroup() %>%
  mutate(perc = n/sum(n)) %>%
  pull(perc)

n_it_G<- nlopt1$iterations
n_it_L<- nlopt2$iterations

# make output report
perc <- scales::label_percent(accuracy = 0.01)
report <- glue(
  "CHINOOK CALIBRATION REPORT

  Calibration completed at {start_time2+elapsed_time2}.
  Calibration ran for {n_it_G} iterations (global algorithm)
  followed by {n_it_L} iterations (local algorithm)
  Maximum allowed iterations: {nRunGlobal} global, {nRunLocal} local.
  Total time was {round(elapsed_total)} minutes

  Target subyearling SAR:  {perc(SAR_target)}
  Modeled subyearling SAR: {perc(SAR_model)}

  Target yearling returns:  {perc(subyearling_target[2])}
  Modeled yearling returns: {perc(yearlings)}

  Target outmigrant sizes (fry, parr):  {toString(perc(fry_vector))}
  Modeled outmigrant sizes (fry, parr): {toString(perc(outmigrants))}

  Modeled spawners: {round(spawners_model)}
  Modeled spawners with 30% harvest: {round(spawners_harvest)}

  CALIBRATED VALUES:
  {toString(round(results2, 9))}

  subyr_ratio = {round(results2[1], 9)},
  ps_parr_surv = {round(results2[2], 9)},
  fry_migr_prod = {round(results2[3], 9)}"
)

writeLines(
  report, sprintf("ignore/STL_chinook_calibration_%s.txt", Sys.Date()))
