# calibrate SAR and age structure for steelhead
devtools::load_all(export_all = F)
library(tidyverse)

out_dir <-
  "//nwcfile.nmfs.local/FE/watershed/SnoStilly_lcm/figures/diagnostic"

basin <- "STL"

# calibration targets
fry_ratio_target <- 26.8/73.2 # ratio from smolt trap
fry_vector <- c(0.268, 0.732)
SAR_target <- 0.012  # target SAR
subyearling_target <- c(0.99, 0.01) # yearlings avg 1%

chinook_base <- hab_run("fall_chinook", "Current", "curr") %>% lcm_assign()

# how many spawners do we have with default parameters?
chinook_base %>%
  lcm_run() %>%
  filter(year == 100) %>%
  summarize(n = sum(n))

# use fry ratio in RMSE
calc_everything <- function(x, dprod){
  all_lcm <- chinook_base %>%
    mutate(
      subyr_ratio = x[1],
      ps_parr_surv = x[2],
      fry_migr_prod = x[3],
      delta_prod = dprod
    )  %>%
    lcm_run("all") %>%
    filter(year == 100)

  fry_ratio <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = fry_ratio_target) %>%
    mutate(
      fry_ratio = fry_migrants/parr_migrants,
      fry_diff = ((fry_ratio - target)/100)^2) %>%
    pull(fry_diff)

  parr_SAR <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants", "total_run_subyr")) %>%
    mutate(
      lifestage = ifelse(grepl("migrants", lifestage), "smolt", "adult")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = c(SAR_target)) %>%
    mutate(
      SAR = adult/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  subyr <- all_lcm %>%
    filter(
      grepl("total_run_", lifestage)) %>%
    mutate(
      fw_age = stringr::str_extract(lifestage, "(?<=total_run_)\\w+"),
    ) %>%
    group_by(fw_age) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    ungroup() %>%
    mutate(perc = n/sum(n)) %>%
    filter(fw_age == "yr") %>% # match yearlings (matching both is redundant)
    cbind(target = subyearling_target[2])%>%
    mutate(sq_diff = ((perc-target)/10)^2) %>%
    pull(sq_diff)

  sqrt(sum(fry_ratio, parr_SAR*2, subyr)/4) #weight SARs 2x that of the ages
}

calibrate_each_delta <- function(dprod){
  nlopt1 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = c(0.98,
           0.056,
           0.10),
    eval_f = calc_everything,
    lb = rep(0,3),
    ub = rep(1,3),
    dprod = dprod,
    opts = list(
      algorithm ="NLOPT_GN_DIRECT_L",
      print_level = 1,
      maxeval = 500
    )
  )
  results1 <- nlopt1$solution # store results of global optimization
  nlopt2 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = results1, # start with global results
    eval_f = calc_everything,
    lb = rep(0,3),
    ub = rep(1,3),
    dprod = dprod,
    opts = list(
      algorithm ="NLOPT_LN_COBYLA",
      print_level = 1,
      maxeval = 500
    )
  )
  results2 <- nlopt2$solution

  c(results2, delta_prod = dprod)
}

cal_out <- map_dfr(seq(0.1,0.5,0.1), calibrate_each_delta)

cal_ren <- rename(cal_out, syr = `...1`, psps = `...2`, fmp = `...3`)

# make four different restoration senarios
chin_curr <- chinook_base # current

# full restoration
chin_full <- hab_run("fall_chinook", "All_100%", "curr") %>% lcm_assign()

# full freshwater restoration
chin_fw   <- chin_full %>%
  mutate(capacity_delta = harp:::params_hab_fall_chinook$delta_cap_curr)

# only delta restoration
chin_delta<- chinook_base %>%
  mutate(capacity_delta = harp:::params_hab_fall_chinook$delta_cap_hist)

# function to apply calibration to given scenario
apply_cal <- function(hab_data, params){
  hab_data %>%
    mutate(
      subyr_ratio = params$syr,
      ps_parr_surv = params$psps,
      fry_migr_prod = params$fmp,
      delta_prod = params$delta_prod) %>%
    lcm_run()  %>%
    filter(year == 100) %>% pull(n) %>% sum()
}

# cal_ren<-readRDS("ignore/calibration_constants_delta_STL_0818.RDS")


apply_each_cal <- function(hab_data, sc_name){
  map_dfr(split(cal_ren, cal_ren$delta_prod), # this is terrible
          ~c(spawners = apply_cal(hab_data, .x),
             dprod = .x$delta_prod,
             scenario = sc_name))
}

spawners <- bind_rows(
  apply_each_cal(chinook_base, "No Action"),
  apply_each_cal(chin_delta, "Delta Restoration"),
  apply_each_cal(chin_fw, "Freshwater Restoration"),
  apply_each_cal(chin_full, "Full Restoration")) %>%
  mutate(
    spawners = as.numeric(spawners),
    dprod = as.numeric(dprod),
    scenario = factor(
      scenario,
      levels = c("No Action", "Delta Restoration",
                 "Freshwater Restoration", "Full Restoration")))
# plot returners
spawners %>%
  ggplot()+
  geom_point(
    aes(dprod, spawners, color = scenario))+
  geom_line(
    linetype = 2,
    aes(dprod, spawners, color = scenario, group = scenario))+
  scale_x_continuous(
    name = "Delta Productivity",
    labels = scales::percent
  )+
  scale_y_continuous(
    name = "Returners (No Harvest)",
    labels = scales::comma
  )+
  scale_color_discrete(name = "Scenario")+
  theme_bw()

ggsave(
  file.path(
    out_dir,
    sprintf("delta_productivity_Neq_%s_%s.png", basin, Sys.Date())),
  width = 6, height = 4)

spawners %>%
  group_by(dprod) %>%
  mutate(
    n_curr = spawners[scenario == 'No Action'],
    n_change = spawners - n_curr,
    perc_change = n_change/n_curr
  ) %>%
  ggplot()+
  geom_point(
    aes(dprod, perc_change, color = scenario))+
  geom_line(
    linetype = 2,
    aes(dprod, perc_change, color = scenario, group = scenario))+
  scale_x_continuous(
    name = "Delta Productivity",
    labels = scales::percent
  )+
  scale_y_continuous(
    name = "Percent Change from Current",
    labels = scales::percent
  )+
  scale_color_discrete(name = "Scenario")+
  theme_bw()

ggsave(
  file.path(
    out_dir,
    sprintf("delta_productivity_perc_change_%s_%s.png", basin, Sys.Date())),
  width = 6, height = 4)
