# calibrate SAR and age structure for steelhead
devtools::load_all(export_all = F)
library(tidyverse)
library(glue)

# set calibration targets
wSAR <- 0.016
sSAR <- 0.016
wAges <- c(0.1045, 0.7825, 0.1130)
sAges <- c(0.0168, 0.7983, 0.1849)
nRunGlobal <- 2000
nRunLocal <- 2000


# set up current scenario habitats
steelhead_hab_w <- hab_run('winter_steelhead', 'Current', 'curr')
steelhead_hab_s <- hab_run('summer_steelhead', 'Current', 'curr')

steelhead_base <- lcm_assign_steelhead(steelhead_hab_w, steelhead_hab_s)

# check current escapement
steelhead_base %>%
  #mutate(S_up_w = 1, S_up_s = 1) %>%
  lcm_run() %>%
  filter(
    year == 100,
    subbasin %in% c(
      "mainstem_North_Fork_Stillaguamish_03",
      "mainstem_North_Fork_Stillaguamish_02",
      "mainstem_North_Fork_Stillaguamish_01"
    )) %>%
  summarize(n = sum(n))

steelhead_base %>%
  #mutate(S_up_w = 1, S_up_s = 1) %>%
  lcm_run() %>%
  filter(year == 100) %>%
  summarize(n = sum(n))

calc_steelhead_diffs <- function(x){
  lcm_results <- steelhead_base %>%
    mutate(
      prop_age1_smolts_w = x[1],
      prop_age2_smolts_w = x[2],
      prop_age1_smolts_s = x[3],
      prop_age2_smolts_s = x[4],
      bay_surv_w = x[5],
      bay_surv_s = x[6]
    )  %>%
    lcm_run("all")

  fw_ages <- lcm_results %>%
    filter(
      year == 100,
      grepl("_spawn_", lifestage)) %>%
    mutate(
      fw_age = stringr::str_extract(lifestage, "(?<=age)\\d"),
      run = stringr::str_extract(lifestage, "(?<=spawn_)\\w+"),
    ) %>%
    group_by(fw_age, run) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    group_by(run) %>%
    mutate(perc = n/sum(n)) %>%
    arrange(desc(run)) %>% # winters first
    cbind(target = c(
      wAges, # winters
      sAges))%>% # then summers
    mutate(
      sq_diff = (perc-target)^2
    ) %>%
    pull(sq_diff) %>%
    sum()

  sar <- lcm_results %>%
    filter(
      year == 100,
      lifestage %in% c(
        "total_run_summer", "total_run_winter",
        "smolts_summer", "smolts_winter")) %>%
    mutate(
      run = stringr::str_extract(lifestage, "(?<=_)[:alpha:]+$"),
      lifestage = stringr::str_extract(lifestage, "\\w+(?=_)")) %>%
    group_by(run, lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    arrange(desc(run)) %>%
    cbind(target = c(wSAR, sSAR)) %>%
    mutate(
      SAR = total_run/smolts,
      SAR_diff = ((SAR - target)*10)^2) %>%
    pull(SAR_diff) %>%
    sum()

  sqrt((fw_ages+sar*6)/12) # weight SARs equal to ages
}

# minimize the overall function with nonlinear optimizer
start_time <- Sys.time()
opt_results <- nloptr::nloptr( # run nonlinear optimizer
  x0 = c( # starting point estimations for optimization
    0.033,
    0.72,
    0.005,
    0.63,
    0.017,
    0.028),
  eval_f = calc_steelhead_diffs,
  lb = rep(0,6), # lower bound for each value is 0
  ub = rep(1,6), # upper bound for each value is 1
  opts = list(
    algorithm ="NLOPT_GN_DIRECT_L", # global optimization
    print_level = 3, #print progress
    maxeval = nRunGlobal # 2000 iterations
  )
)
elapsed_time <- (Sys.time() - start_time) # print elapsed time
print(elapsed_time)

stored_par <- opt_results$solution

# check final RMSE
calc_steelhead_diffs(stored_par)

start_time2 <- Sys.time()
opt_results2 <- nloptr::nloptr( # run nonlinear optimizer
  x0 = stored_par,
  eval_f = calc_steelhead_diffs,
  lb = rep(0,6), # lower bound for each value is 0
  ub = rep(1,6), # upper bound for each value is 1
  opts = list(
    algorithm ="NLOPT_LN_COBYLA", # global optimization
    print_level = 3, #print progress
    maxeval = nRunLocal # 2000 iterations
  )
)
elapsed_time2 <- (Sys.time() - start_time2) # print elapsed time
print(elapsed_time2)

elapsed_total <- as.numeric(elapsed_time+elapsed_time2, units = "hours")

stored_par2 <- opt_results2$solution

# run LCM with calibration values
calibrated_results <- steelhead_base %>%
  mutate(
    prop_age1_smolts_w = stored_par2[1],
    prop_age2_smolts_w = stored_par2[2],
    prop_age1_smolts_s = stored_par2[3],
    prop_age2_smolts_s = stored_par2[4],
    bay_surv_w = stored_par2[5],
    bay_surv_s = stored_par2[6]
  ) %>%
  lcm_run("all") %>%
  filter(year == 100)

# extract model results -------------------------------------------------------
spawners_winter <- calibrated_results %>%
  filter(lifestage == "spawners_winter") %>%
  summarize(n = sum(n))

spawners_winter_index <- calibrated_results %>%
  filter(
    lifestage == "spawners_winter",
    subbasin %in% c(
      "mainstem_North_Fork_Stillaguamish_02",
      "mainstem_North_Fork_Stillaguamish_03",
      "mainstem_North_Fork_Stillaguamish_04"
    )) %>%
  summarize(n = sum(n))

spawners_summer <- calibrated_results %>%
  filter(lifestage == "spawners_summer") %>%
  summarize(n = sum(n))

age_structure <- calibrated_results %>%
  filter(grepl("_spawn_", lifestage)) %>%
  mutate(
    fw_age = stringr::str_extract(lifestage, "(?<=age)\\d"),
    run = stringr::str_extract(lifestage, "(?<=spawn_)\\w+"),
  ) %>%
  group_by(fw_age, run) %>%
  summarize(n = sum(n), .groups = "keep") %>%
  group_by(run) %>%
  mutate(perc = n/sum(n)) %>%
  arrange(desc(run)) %>% pull(perc)

sar <- calibrated_results %>%
  filter(lifestage %in% c(
    "total_run_summer", "total_run_winter",
    "smolts_summer", "smolts_winter")) %>%
  mutate(
    run = stringr::str_extract(lifestage, "(?<=_)[:alpha:]+$"),
    lifestage = stringr::str_extract(lifestage, "\\w+(?=_)")) %>%
  group_by(run, lifestage) %>%
  summarize(n = sum(n), .groups = "keep") %>%
  pivot_wider(names_from = lifestage, values_from = n) %>%
  arrange(desc(run)) %>%
  cbind(target = c(wSAR, sSAR)) %>%
  mutate(
    SAR = total_run/smolts) %>%
  pull(SAR)

n_it_G<- opt_results$iterations
n_it_L<- opt_results2$iterations

# make output report
perc <- scales::label_percent(accuracy = 0.01)
report <- glue(
  "STEELHEAD CALIBRATION REPORT

  Calibration completed at {start_time2+elapsed_time2}.
  Calibration ran for {n_it_G} iterations (global algorithm)
  followed by {n_it_L} iterations (local algorithm)
  Maximum allowed iterations: {nRunGlobal} global, {nRunLocal} local.

  Total time was {round(elapsed_total, 2)} hours

  Target winter SAR: {perc(wSAR)}
  Target summer SAR: {perc(sSAR)}

  Modeled winter SAR: {perc(sar[1])}
  Modeled summer SAR: {perc(sar[2])}

  Target winter age structure (% return FW age 1-3): {toString(perc(wAges))}
  Target summer age structure (% return FW age 1-3): {toString(perc(sAges))}

  Modeled winter age structure: {toString(perc(age_structure[1:3]))}
  Modeled summer age structure: {toString(perc(age_structure[4:6]))}

  Modeled winter-run spawners (index area): {round(spawners_winter_index)}
  Modeled winter-run spawners (whole basin): {round(spawners_winter)}
  Modeled summer-run spawners (whole basin): {round(spawners_summer)}

  CALIBRATED VALUES:
  {toString(round(stored_par2, 9))}

  prop_age1_smolts_w = {round(stored_par2[1], 9)},
  prop_age2_smolts_w = {round(stored_par2[2], 9)},
  prop_age1_smolts_s = {round(stored_par2[3], 9)},
  prop_age2_smolts_s = {round(stored_par2[4], 9)},
  bay_surv_w = {round(stored_par2[5], 9)},
  bay_surv_s = {round(stored_par2[6], 9)}"

)

writeLines(
  report, sprintf("ignore/STL_steelhead_calibration_%s.txt", Sys.Date()))
