devtools::load_all()
library(ggplot2)
library(tidyverse)

server_dir <-
  "//nwcfile.nmfs.local/FE/watershed/SnoStilly_lcm/figures/sensitivity"

temp_save_dir <- "ignore/sensitivity"

# lifestages and groupings
# table to recode parameters
code_table <- tribble(
  ~pop,           ~param,               ~lifestage,       ~type,          ~grp,
  "coho",         "egg_cap",            "Spawning",       "Capacity",     "sp",
  "coho",         "egg_fry_surv",       "Incubation",     "Productivity", "sp",
  "coho",         "parr_surv",          "Summer",         "Productivity", "s",
  "coho",         "parr_cap",           "Summer",         "Capacity",     "s",
  "coho",         "parr_smolt_surv",    "Winter",         "Productivity", "w",
  "coho",         "parr_smolt_cap",     "Winter",         "Capacity",     "w",
  "coho",         "S_up",               "Prespawn",       "Productivity", "sp",
  "fall_chinook", "egg_cap",            "Spawning",       "Capacity",     "sp",
  "fall_chinook", "egg_fry_surv",       "Incubation",     "Productivity", "sp",
  "fall_chinook", "spring_cap",         "Subyearling",    "Capacity",     "s",
  "fall_chinook", "colnz_surv",         "Colonization",   "Productivity", "s",
  "fall_chinook", "spring_surv",        "Subyearling",    "Productivity", "s",
  "fall_chinook", "summer_cap",         "Summer",         "Capacity",     "s2",
  "fall_chinook", "summer_surv",        "Summer",         "Productivity", "s2",
  "fall_chinook", "capacity_w",         "Winter",         "Capacity",     "w",
  "fall_chinook", "winter_surv",        "Winter",         "Productivity", "w",
  #"fall_chinook", "capacity_delta",     "Delta",          "Capacity",     "d",
  "fall_chinook", "S_up",               "Prespawn",       "Productivity", "sp",
  "steelhead",    "egg_cap_s",          "Spawning (SR)",  "Capacity",     "sp_sr",
  "steelhead",    "egg_cap_w",          "Spawning (WR)",  "Capacity",     "sp_wr",
  "steelhead",    "egg_fry_surv_s",     "Incubation (SR)","Productivity", "sp_sr",
  "steelhead",    "egg_fry_surv_w",     "Incubation (WR)","Productivity", "sp_wr",
  "steelhead",    "parr_cap",           "Summer",         "Capacity",     "s",
  "steelhead",    "parr_surv",          "Summer",         "Productivity", "s",
  "steelhead",    "first_winter_cap",   "Winter",         "Capacity",     "w",
  "steelhead",    "first_winter_surv",  "Winter",         "Productivity", "w",
  "steelhead",    "second_summer_cap",  "2nd Summer",     "Capacity",     "s2",
  "steelhead",    "second_summer_surv", "2nd Summer",     "Productivity", "s2",
  "steelhead",    "second_winter_cap",  "2nd Winter",     "Capacity",     "w2",
  "steelhead",    "second_winter_surv", "2nd Winter",     "Productivity", "w2",
  "steelhead",    "S_up_s",             "Prespawn (SR)",  "Productivity", "sp_sr",
  "steelhead",    "S_up_w",             "Prespawn (WR)",  "Productivity", "sp_wr"
)

# function to get capacity-weighted averages from lcm_assign output
weight_basin <- function(x){ # x is input table
  hab_cols <- 12:ncol(x)
  x %>%
    pivot_longer( # make long
      all_of(hab_cols),
      names_to = "param",
      values_to = "value") %>%
    left_join(code_table, by = c("pop", "param")) %>% # class lifestages
    filter(!is.na(lifestage)) %>%
    group_by(pop, subbasin_num, grp) %>%
    mutate(
      weights = value[type == "Capacity"], # get capacities
      value = case_when( # transform weekly values back to seasonal ones
        param == "colnz_surv" ~ value^11,
        param == "capacity_delta" ~ value/n(),
        TRUE ~ value)
    ) %>%
    group_by(pop, param, lifestage, type) %>%
    summarize(
      w_mean = weighted.mean(value, weights), # weighted mean
      basin_sum = sum(value), # basin sum
      .groups = "keep") %>%
    mutate(stat = ifelse(type == "Productivity", w_mean, basin_sum)) %>%
    select(pop, param, lifestage, type, stat)
}

# function to get current and historic habitat data
get_hab <- function(pop){

  # run habitat models ---------------------------------------------------------
  if(pop == "steelhead"){
    h_curr_assign <- lcm_assign_steelhead(
      hab_run("winter_steelhead", 'Current', "curr"),
      hab_run("summer_steelhead", 'Current', "curr")
    )
    h_hist_assign  <- lcm_assign_steelhead(
      hab_run("winter_steelhead", 'Historical', "hist"),
      hab_run("summer_steelhead", 'Historical', "hist")
    )
  } else {
    # get current era habitat
    h_curr_assign <- hab_run(pop, 'Current', "curr") %>% # run current conditions
      lcm_assign()
    h_hist_assign  <- hab_run(pop, 'Historical', "hist")%>% # run historic
      lcm_assign()
  }

  # get subbasin-scale differences between eras --------------------------------

  hab_cols <- 12:ncol(h_curr_assign) # how many habitat cols are there?

  h_curr <- h_curr_assign %>% # format current hab data for lcm input
    pivot_longer(
      all_of(hab_cols),
      names_to = "param",
      values_to = "value_curr")

  h_hist <- h_hist_assign %>% # and historic data
    pivot_longer(
      all_of(hab_cols),
      names_to = "param",
      values_to = "value_hist") %>%
    select(subbasin_num, param, value_hist)

  # compare current and historic, return this df
  subbasin_diffs <- inner_join(
    h_curr, h_hist, by = c("subbasin_num", "param")) %>%
    ungroup() %>%
    mutate(vdiff = value_hist-value_curr)

  # get basin-scale differences ------------------------------------------------
  h_curr_basin <- weight_basin(h_curr_assign)
  h_hist_basin <- weight_basin(h_hist_assign)

  basin_diffs <- inner_join(
    h_curr_basin, h_hist_basin,
    by = c("pop", "param", "lifestage", "type")) %>%
    rename(v_curr = "stat.x", v_hist = "stat.y") %>%
    ungroup() %>%
    mutate(rest_potential = (v_hist - v_curr)/v_curr)

  list(
    subba = subbasin_diffs,
    basin = basin_diffs
  )}

# generic function to run sensitivity analysis with habitat input
run_generic <- function(hab_data, pop, mparam, adj){
  # clean up lcm input table
  lcm_input <- hab_data %>%
    mutate( # select value to change and change it
      value = ifelse(param == mparam, value_curr + (vdiff * adj), value_curr)) %>%
    select(-c(value_hist, value_curr, vdiff)) %>% # clean up
    pivot_wider(names_from = param, values_from = value) # make wide again

  # calculate spawner abundance
  lcm_run(lcm_input) %>%
    filter(year == 100) %>% # change for stochastic version
    summarize(n = sum(n)) %>%
    mutate(
      pop = pop,
      param = mparam,
      adj = adj)
}

# function to run simulation for one species
run_simulation <- function(hab_data){ # takes a list of hab data
  # find out which params change from current to historic for this pop
  mutable_params <- hab_data[["basin"]] %>%
    transmute(pop, mparam = param)

  # table of what to vary and by how much
  adjustment_table <- expand_grid(mutable_params, adj = seq(0, 1, 0.1))

  # function to run lcm with single adjusted parameter
  run_specific <- function(pop, mparam, adj){
    run_generic(hab_data[["subba"]], pop, mparam, adj) %>%
      left_join(hab_data[["basin"]], by = c("pop", "param")) %>%
      mutate(percent_change = rest_potential*adj)
  }

  pmap_dfr(adjustment_table, run_specific) # function uses named parameters
}

# function to put everything together
run_one_pop <- function(pop){
  get_hab(pop) %>%
    run_simulation()
}

# run all of the pops
all_runs <- map_dfr(c("coho", "steelhead", "fall_chinook"), run_one_pop)

df_plt <- all_runs %>% # prepare data frame for plotting
  mutate(pop = factor(
    pop,
    levels = c("coho", "fall_chinook", "steelhead"),
    labels = c("Coho", "Chinook", "Steelhead"))) %>%
  filter(!(lifestage == "Winter" & pop == "Chinook" & type == "Capacity")) %>%
  na.omit()

df_endpoints <- df_plt %>%
  filter(adj == 1) %>%
  mutate(lab = lifestage) %>%
  na.omit() %>%
  mutate(scolor = ifelse(type == "Capacity", "#F8766D", "#00BFC4"))

if(!dir.exists(temp_save_dir)) dir.create(temp_save_dir)
saveRDS(df_endpoints,file.path(temp_save_dir, "df_endpts.RDS"))
saveRDS(df_plt,file.path(temp_save_dir, "df_plt.RDS"))

sens_plot <- df_plt %>%
  ggplot(aes(
    percent_change,
    n,
    color = type,
    group = param
  )) +
  facet_wrap(pop~., scales = 'free', nrow = 3) +
  cowplot::theme_half_open() +
  geom_line(size = 0.8) +
  geom_point(data = df_endpoints) +
  ggrepel::geom_label_repel(
    data = df_endpoints,
    aes(label = lab, segment.color = scolor),
    nudge_x = Inf,
    direction = "y",
    hjust = 0.7, # vertical position is sensitive to hjust, unclear why
    size = 3.2, # shrink labels to fit
    segment.size = 0.3,
    segment.linetype = 2,
    segment.curvature = -0.3,
    segment.inflect = TRUE,
    show.legend = FALSE
  )+
  scale_x_continuous(
    expand = expansion(mult = c(0.05, 0.6)), # expand x axis to fit labels
    labels = scales::label_percent()
  ) +
  scale_y_continuous(labels = scales::label_comma()) +
  labs(y = 'Spawners',
       x = 'Restoration benefit',
       color = NULL, title = "Stillaguamish") +
  theme(legend.position = c(0, 0.98),
        plot.background = element_rect(fill = "white"),
        panel.spacing.y = unit(-0.02, "inches"),
        panel.grid.major = element_line('grey90'),
        plot.title = element_text(size = 13, face = "plain", hjust = 0.5),
        strip.background = element_rect(fill = NA))

ggsave(
  file.path(temp_save_dir, sprintf("all_sensitivity_STL_%s.png", Sys.Date())),
  width = 4,
  height = 12)

if(dir.exists(server_dir)){
  ggsave(
    file.path(server_dir, sprintf("all_sensitivity_STL_%s.png", Sys.Date())),
    width = 4,
    height = 12)
}

