library(tidyverse)
load_all()

width_df <- harp::flowline_clean %>% # start with the base flowling
  select(noaaid, BF_width) %>%
  distinct() %>%
  left_join(harp::width_df, by = "noaaid") %>%
  mutate(is_small = BF_width < 20)

vary_width <- function(pop, width_mult){ # vary summer and winter together
  width_mod <- width_df %>%
    transmute(
      noaaid,
      # only adjust small streams, cap expansion at 30 m
      width_s = pmin(width_s * (1 + width_mult * is_small), 30),
      width_w = pmin(width_w * (1 + width_mult * is_small), 30)
    )

  hab_run(pop, "Current", "curr", df_width = width_mod)
}

vary_width("coho", 0) %>%
  lcm_assign() %>% lcm_run() %>% filter(year == 100) %>% summarize(n = sum(n))

vary_width("coho", 0.5) %>%
  lcm_assign() %>% lcm_run() %>% filter(year == 100) %>% summarize(n = sum(n))

sum_spawners_width <- function(pop, width_mult){
  suppressMessages(vary_width(pop, width_mult)) %>%
    lcm_assign() %>% lcm_run() %>%
    filter(year == 100) %>%
    summarize(n = sum(n)) %>%
    mutate(pop = pop, width_mult = width_mult)
}

sum_spawners_width_stlhd <- function(width_mult){
  suppressMessages(
    lcm_assign_steelhead(
      vary_width("winter_steelhead", width_mult),
      vary_width("summer_steelhead", width_mult))) %>%
    lcm_run() %>%
    filter(year == 100) %>%
    summarize(n = sum(n)) %>%
    mutate(pop = "steelhead", width_mult = width_mult)
}

width_results <- crossing(seq(0, 3, 0.1), c("coho", "fall_chinook")) %>%
  pmap_dfr(~sum_spawners_width(..2, ..1))

width_results_stlhd <- map_dfr(
  seq(0, 3, 0.1),
  ~sum_spawners_width_stlhd(.x))

escapement <- data.frame(
  pop = c("Chinook", "Coho", "Steelhead"),
  n = c(982, 19495, NA),
  observed = "Median Esc.",
  SAR = c(0.11,0.041,NA))

bind_rows(width_results, width_results_stlhd) %>%
  mutate(pop = case_when(
    pop == "coho" ~ "Coho",
    pop == "fall_chinook" ~ "Chinook",
    pop == "steelhead" ~ "Steelhead"
  )) %>%
  ggplot()+
  geom_hline(aes(yintercept = n, color = observed), data = escapement)+
  geom_vline(
    xintercept = 0,
    data = escapement,
    color = "gray50", linetype = 2)+
  geom_line(aes(width_mult, n))+
  facet_wrap("pop", scales = "free")+
  cowplot::theme_half_open() +
  scale_x_continuous(
    labels = scales::label_percent()
  )+
  labs(y = 'Spawners',
       x = 'Width Increase',
       color = NULL, title = "Stillaguamish")+
  theme(panel.grid.major = element_line('grey90'),
        strip.background = element_rect(fill = NA),
        legend.position = "bottom")

ggsave(
  sprintf("ignore/results/spawners_v_width_STL_%s.png", Sys.Date()),
  width = 8,
  height = 4.5)
