library(tidyverse)
load_all()

width_df <- harp::flowline_clean %>% # start with the base flowling
  select(noaaid, BF_width) %>%
  distinct() %>%
  left_join(harp::width_df, by = "noaaid") %>%
  mutate(is_small = BF_width < 20)

vary_width <- function(pop, seed){ # vary summer and winter together
  set.seed(seed) # needed for steelhead
  width_mod <- width_df %>%
    transmute(
      noaaid,
      width_mult = runif(n(), -0.5, 0.5), # uniform dist
      # only adjust small streams, cap expansion at 30 m
      width_s = pmin(width_s * (1 + width_mult * is_small), 30),
      width_w = pmin(width_w * (1 + width_mult * is_small), 30)
    )

  hab_run(pop, "Current", "curr", df_width = width_mod)
}

vary_width_nocap <- function(pop, seed){ # vary summer and winter together
  set.seed(seed) # needed for steelhead
  width_mod <- width_df %>%
    transmute(
      noaaid,
      width_mult = runif(n(), -0.5, 0.5), # uniform dist
      # only adjust small streams, cap expansion at 30 m
      width_s = width_s * (1 + width_mult * is_small),
      width_w = width_w * (1 + width_mult * is_small)
    )

  hab_run(pop, "Current", "curr", df_width = width_mod)
}

sum_spawners_width <- function(pop, seed){
  #suppressMessages(vary_width_nocap(pop, seed)) %>%
  suppressMessages(vary_width(pop, seed)) %>%
    lcm_assign() %>% lcm_run() %>%
    filter(year == 100) %>%
    summarize(n = sum(n)) %>%
    mutate(pop = pop, seed = seed)
}

sum_spawners_width_stlhd <- function(seed){
  suppressMessages(
    lcm_assign_steelhead(
      vary_width("winter_steelhead", seed),
      vary_width("summer_steelhead", seed))) %>%
    lcm_run() %>%
    filter(year == 100) %>%
    summarize(n = sum(n)) %>%
    mutate(pop = "steelhead", seed = seed)
}

width_results <- crossing(seq(1, 1000), c("coho", "fall_chinook")) %>%
  pmap_dfr(~sum_spawners_width(..2, ..1))

width_results_stlhd <- map_dfr(
  seq(1, 1000),
  ~sum_spawners_width_stlhd(.x))

bind_rows(width_results, width_results_stlhd) %>%
  saveRDS("ignore/results/spawners_v_width_stoch_STL_%s.RDS")

bind_rows(width_results, width_results_stlhd) %>%
  mutate(pop = case_when(
    pop == "coho" ~ "Coho",
    pop == "fall_chinook" ~ "Chinook",
    pop == "steelhead" ~ "Steelhead"
  )) %>%
  ggplot()+
  geom_histogram(aes(n), bins = 15, fill="gray70")+
  facet_wrap("pop", scales = "free_x")+
  cowplot::theme_half_open() +
  # scale_x_continuous(
  #   labels = scales::label_percent()
  # )+
  labs(y = 'Model Runs',
       x = 'Spawners',
       color = NULL, title = "Stillaguamish variable wetted width")+
  theme(panel.grid.major = element_line('grey90'),
        strip.background = element_rect(fill = NA),
        axis.text.x = element_text(angle = 90))

ggsave(
  sprintf("ignore/results/spawners_v_width_stoch_STL_%s.png", Sys.Date()),
  width = 8,
  height = 4.5)
