library(tidyverse)
library(cowplot)

# setup base steelhead habitat run
steelhead_hab <- lcm_assign_steelhead(
  hab_run('winter_steelhead', 'Current', 'curr'),
  hab_run('summer_steelhead', 'Current', 'curr'))

# function to make initial seeding of summer and winter steelhead more similar
converge_seed <- function(seed_mult){

  steelhead_hab %>%
    mutate(
      pop_init_mean = (pop_init_s+pop_init_w)/2,
      both_runs = pmin(pop_init_s, pop_init_w) > 0,
      pop_init_w = case_when(
        both_runs ~ seed_mult*pop_init_mean+(1-seed_mult)*pop_init_w,
        TRUE ~ pop_init_w),
      pop_init_s = case_when(
        both_runs ~ seed_mult*pop_init_mean+(1-seed_mult)*pop_init_s,
        TRUE ~ pop_init_s
      )) %>%
    lcm_run(c("spawners_winter", "spawners_summer")) %>%
    #filter(year == 100) %>%
    left_join(harp:::subbasin_names, by = c(subbasin = "noaa_subba")) %>%
    group_by(subb_group, lifestage, year) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    mutate(
      year = as.numeric(year),
      trans = "conv",
      value = seed_mult,
      percent_winters = spawners_winter/(spawners_winter+spawners_summer))
}


# run models with variety of parameters,
# get df out for each type of modification
steelhead_seed_mult <- map_df(seq(0,1,0.1), converge_seed)


# base function plot
plot_steels <- function(df, x){
  ggplot(df, aes(value, percent_winters, color = year))+
    #geom_point(size = 1)+
    geom_line()+
    theme_cowplot()+
    theme(axis.title.y = element_text(color = "gray"))+
    labs(y = "Basinwide Spawners", x = x)+
    scale_x_continuous(labels = scales::percent_format())
}

# plot each
plot_steels(steelhead_seed_mult, "Seed convergence")+
  facet_wrap(subb_group~.)

steelhead_seed_mult %>%
  filter(value %in% c(0, 0.5, 1), subb_group == "South_Fork") %>%
  ggplot(aes(year, percent_winters, color = value, group = value))+
  #geom_point(size = 1)+
  geom_line()+
  theme_cowplot()+
  theme(axis.title.y = element_text(color = "gray"))+
  labs(y = "Percent winters", x = "Years", color = "Seed\nEquality")+
  scale_y_continuous(labels = scales::percent_format())+
  facet_wrap(subb_group~.)

# combine plots
plot_grid(
  plotlist = list(p1, p2, p3, p4)) %>%
  save_plot(
    sprintf("ignore/steelhead_migration_diagnostics_%s.png", Sys.Date()),
    .,
    base_height = 5)
