devtools::load_all()
library(dplyr)
library(tidyr)

# save directory
save_dir <- "~/lcm_outputs"
basin_name <- "Stillaguamish"
basin_prefix <- "STL"

# flowline-to-subbasin table
fl2s_df <- harp::flowline_clean %>%
  distinct(noaaid, subbasin_num) %>%
  left_join(subbasin_names, by = "subbasin_num") %>%
  select(noaaid, subbasin_num, noaa_subba)

# export historical and current temperature
temp_wide <- temperature_df %>%
  filter(era == "curr" & scenario == "Current" |
           era == "hist" & scenario == "Historical") %>%
  select(-c(scenario, temp_prespawn)) %>%
  pivot_wider(names_from = era, values_from = c(temp_Jn121, temp_7DADM))

# get egg-to-fry survival
e2f <- flowline_clean %>%
  filter(rear) %>%
  select(noaaid, pop, ef_surv_current, ef_surv_hist) %>%
  mutate(basin = basin_name)

# get total length by pop and subbasin
river_kms <- flowline_clean %>%
  group_by(pop, subbasin_num, noaa_subba) %>%
  summarize(
    rear_km = sum(rear*length)/1000,
    spawn_km = sum(spawn*length)/1000) %>%
  mutate(basin = basin_name)


# any rearing habitat
rearing_reaches <- flowline_clean %>%
  filter(rear) %>%
  pull(noaaid) %>%
  unique()

# barriers by subbasin
barriers_by_subb <- flowline_clean %>%
  select(noaaid, noaa_subba, subbasin_num, length) %>%
  distinct() %>%
  filter(noaaid %in% rearing_reaches) %>%
  left_join(filter(barriers_df, scenario == "Current")) %>%
  mutate(
    pass_class = case_when(
      pass_tot_scenario == 0 ~"Full Barrier",
      pass_tot_scenario == 1 ~ "No Barrier",
      TRUE ~ "Partial Barrier"),
    pass_class = factor( # convert to factor
      pass_class,
      levels = c("Full Barrier", "Partial Barrier", "No Barrier"))) %>%
  group_by(subbasin_num, noaa_subba, pass_class) %>%
  summarize(length = sum(length))


# floodplain habitat
floodplain_formatted <- Floodplain_raw %>%
  filter(noaaid %in% rearing_reaches) %>%
  mutate(bfw_conn = tidyr::replace_na(bfw_conn, FALSE)) %>%
  transmute(
    noaaid,
    Habitat = ifelse(grepl("Pond", Habitat), "Pond", Habitat),
    hist_area = (Period != "Curr")*(in_fp == 1|bfw_conn)*Area_ha,
    curr_area = (Period != "Hist")*(bfw_conn)*Area_ha,
    basin = basin_name) %>%
  arrange(noaaid)

floodplain_by_subb <- floodplain_formatted %>%
  left_join(fl2s_df, by = "noaaid") %>%
  group_by(basin, noaa_subba, subbasin_num, Habitat) %>%
  summarize(across(c(curr_area, hist_area), sum))

# export LR edge types
all_lr <- purrr::map_dfr(
  c(pops_all), # all pops
  ~ hab_run_diagnostic(.x, return_df = "df_lr") # export LR df
) %>%
  filter(Period != "Hist")

lr_combo <- all_lr %>%
  group_by(noaaid, Habitat, pop) %>%
  summarize(length = sum(Length_m)) %>%
  distinct(noaaid, Habitat, length) # don't double-count pops

lr_mod_by_subb <- lr_combo %>%
  left_join(fl2s_df, by = "noaaid") %>%
  mutate(modified = grepl("HM", Habitat)) %>%
  group_by(noaa_subba, subbasin_num, modified) %>%
  summarize(length = sum(length)/1000)

lr_summarize <- lr_combo %>%
  group_by(Habitat) %>%
  summarize(length = sum(length)) %>%
  ungroup() %>%
  mutate(
    percent_total = length/sum(length),
    percent_bank = length*grepl("ank", Habitat)/sum(length*grepl("ank", Habitat))
  )

# compare total lengths, but don't export them
flowline_clean %>%
  filter(Habitat == "LgStream") %>%
  distinct(noaaid, .keep_all = T) %>%
  pull(length) %>%
  sum()
sum(lr_combo$length)/2

export_csv <- function(df, basename){
  write.csv(
    df,
    file.path(save_dir,
              sprintf("%s_%s_%s.csv", basin_prefix, basename, Sys.Date())),
    row.names = FALSE)
}

export_RDS <- function(df, basename){
  saveRDS(
    df,
    file.path(save_dir,
              sprintf("%s_%s_%s.RDS", basin_prefix, basename, Sys.Date())))
}

save_both <- function(df, basename){
  export_csv(df, basename)
  export_RDS(df, basename)
}

save_both(temp_wide, "temperatures")
save_both(e2f, "e2f_surv_by_species")
save_both(floodplain_formatted, "accessible_floodplain_habitats")
save_both(river_kms, "river_kms")
save_both(lr_summarize, "edge_habs_percents")
save_both(lr_mod_by_subb, "mod_lr_bank_by_subb")
save_both(floodplain_by_subb, "accessible_floodplain_habitats_by_subb")
save_both(barriers_by_subb, "barriers_by_subb")
