# Script to run scenarios for diagnostics with full LCM outputs

reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Error checking and directory setup ----

cur_branch <- system2('git', c('branch', '--show-current'), stdout = TRUE)

diffs <- system2('git', c('diff', '--shortstat'), stdout = TRUE)
is_dirty <- !identical(diffs, character(0))

if (cur_branch == 'master') {
  system2('git', c("fetch", "--tags")) # update tags
  cur_tag <- system2('git', c('describe', '--tag'), stdout = TRUE)
  cur_commit <- cur_tag
} else {
  cur_commit <- cur_branch
}

if (is_dirty){
  commit_date <- Sys.Date() # use today for dirty branches
  dir_name <- paste(cur_commit, "dirty", sep = "-")
} else {
  commit_date <-system2('git', c('log', '-1', '--format=%cs'), stdout = TRUE)
  dir_name <- cur_commit
}

# check to make sure package is up to date:
if(packageDate("harp") < commit_date){
  stop("Outdated package version installed.
       Reinstall package for use with furrr: `reinstall_harp()`")
}

# load packages
library(harp)
library(furrr)
library(tidyverse)
plan(multisession, workers = 8)

# prep data directories --------------------------------------------------------
# temporary directory
if (dir.exists('ignore/temp')) unlink('ignore/temp', recursive = TRUE)

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/SnoStilly_lcm/lcm_results/output_archive/STL'

if (cur_branch == 'master' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if (is_dirty) warning('the repo is dirty. Results will not be archived')

if(cur_branch == 'master' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

try(dir.create(out_path, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))

# Run hab_run_stochastic for each combination of pop, scenario, era, run, year -
diag_scenarios <- c(
  "Current",
  "Barriers",
  "Fine_sediment",
  "Wood",
  "Shade",
  "LR",
  "Beaver",
  "Floodplain",
  "Delta")

scenarios <- rbind(
  data.frame(scenario = diag_scenarios, era = "curr"), # diagnostic scenarios
  data.frame(scenario = harp:::scenarios_cust, era = "curr"), # custom ones
  data.frame(scenario = "Historical", era = "hist") # historical
)

# run habitat model for all scenarios
h <- expand_grid(
  pop = harp:::pops_all,
  scenarios) %>%
  pmap_dfr(function(pop, scenario, era){
    if(scenario == "Current") cat(sprintf("Preparing habitat for %s...\n", pop))
    suppressMessages(hab_run(pop, scenario, era))})

saveRDS(
  h,
  file.path(out_path, sprintf('STL_hab_det_%s.rds', commit_date)),
  compress = 'xz')

# Run habitat scenarios through lcm ----

create_lcm <- function(df_hab){
  lcm_assign(df_hab) %>%
    lcm_run("all")
}

l <- h %>%
  filter(pop %in% c("fall_chinook", "coho")) %>%
  group_split(pop, scenario, era) %>%
  future_map_dfr(create_lcm)


create_lcm_steelhead <- function(df_hab_w, df_hab_s){

  lcm_assign_steelhead(df_hab_w, df_hab_s) %>%
    lcm_run(c("all"))
}

l_w_steelhead <- h %>%
  filter(pop == "winter_steelhead") %>%
  group_split(scenario, era)

l_s_steelhead <- h %>%
  filter(pop == "summer_steelhead") %>%
  group_split(scenario, era)

l_steel <- future_map2_dfr(
  l_w_steelhead, l_s_steelhead,
  ~create_lcm_steelhead(.x, .y)
)

saveRDS(
  bind_rows(l, l_steel),
  file.path(out_path, sprintf('STL_lcm_det_%s.rds', commit_date)),
  compress = 'xz')

# Create plots and summary tables ----

top_pal <- harp:::scenario_desc %>%
  filter(scenario %in% diag_scenarios) %>%
  pull(plot_color)

pop_names <-c(
  `coho` = "Coho",
  `fall_chinook` = "Chinook",
  `steelhead` = "Steelhead",
  `wsteelhead` = "Winter Steelhead",
  `ssteelhead` = "Summer Steelhead")

scen_names <-c(
  `Current` = "Current",
  `Barriers` = "No Barriers",
  `Fine_sediment` = "Historical Fine Sediment",
  `Barriers` = "No Manmade Barriers",
  `Wood` = "Historic Wood Loading",
  `Shade` = "Historic Shade",
  `LR` = "Historic Large River Banks and Length",
  `Beaver` = "Historic Beaver Ponds",
  `Floodplain` = "Historic Floodplain Habitat",
  `Delta` = "Historic Delta Habitat")

#reusable code to make barplots
make_barplot <- function(df){
  df %>%
    ggplot() +
    cowplot::theme_half_open() +
    geom_bar(
      aes(scenario, n, fill = scenario),
      color = 'black',
      stat = 'identity') +
    geom_bar(
      aes(scenario, n_cur),
      fill = "gray",
      alpha = 0.8,
      color = 'black',
      stat = 'identity') +
    geom_text(
      aes(scenario,n, label = diff_str),
      vjust = -0.2,
      alpha = 0.8,
      color = 'black') +
    facet_wrap(
      "pop", nrow = 3, scales = "free_y",
      label = as_labeller(pop_names))+
    labs(x = NULL, y = "Spawners", fill = NULL, title = "Stillaguamish") +
    scale_y_continuous(
      labels = scales::comma,
      expand = expansion(c(0, 0.1)))+
    theme(
      panel.grid.major = element_line(color = 'gray90'),
      axis.text.x = element_text(angle = 45, hjust = 1),
      strip.background =element_blank(),
      plot.background = element_rect(fill = "white"),
      legend.position = 'none')}

# function to make barplots of diagnostic scenarios
make_diag_barplot <- function(df){
  df %>%
    filter(
      scenario %in% diag_scenarios, # don't plot historic or non-chinook delta
      !(pop != c("fall_chinook") & scenario == "Delta")) %>%
    make_barplot()+
    scale_fill_manual(values = top_pal) +
    scale_x_discrete(label = as_labeller(scen_names))
}

make_custom_barplot <- function(df){
  df %>%
    filter(
      scenario %in% harp:::scenarios_cust) %>%
    make_barplot()
}

all_pops <- bind_rows(l, l_steel) %>%
  group_by(pop, scenario, era, lifestage, year) %>%
  summarize(n = sum(n)) %>%
  # group_by(pop, scenario, lifestage, era)%>%
  # summarize(n = median(n)) %>%
  filter(
    era == "curr", lifestage == "spawners", year == 100) %>%
  group_by(pop) %>%
  mutate(n_cur = n[scenario == 'Current'],
         diff_perc = (n - n_cur)/n_cur,
         diff_str = scales::percent(
           na_if(diff_perc, 0),
           accuracy = 1, prefix = "+"),
         scenario = factor(
           scenario,
           levels = scenarios$scenario))

# save totals
all_pops %>%
  filter(scenario == "Current") %>%
  ungroup() %>%
  transmute(
    population = pop, scenario, era, N_eq = n, basin = "Stillaguamish") %>%
  write_csv(sprintf("%s/Neq_STL_%s.csv", out_path, commit_date))

all_pops %>%
  mutate(
    pop = factor(pop, levels = c("coho", "steelhead", "fall_chinook"))) %>%
  make_diag_barplot()

ggsave(
  sprintf("%s/barplot_STL_3pops_%s.png", out_path, commit_date),
  width = 6, height = 12)

all_pops %>%
  mutate(
    pop = factor(pop, levels = c("coho", "steelhead", "fall_chinook"))) %>%
  make_custom_barplot()

ggsave(
  sprintf("%s/barplot_STL_3pops_cust_%s.png", out_path, commit_date),
  width = 6, height = 12)

#make and save plots
ind_plots <- all_pops %>%
  ungroup() %>%
  group_split(pop)%>%
  map(function(x){
    pop <- unique(x$pop)
    plot <- make_diag_barplot(x)
    ggsave(sprintf("%s/barplot_STL_%s_%s.png", out_path, pop, commit_date),
           plot = plot,
           width = 6, height = 6)})


l_steel %>%
  group_by(scenario, era, lifestage, year) %>%
  summarize(n = sum(n)) %>%
  # group_by(pop, scenario, lifestage, era)%>%
  # summarize(n = median(n)) %>%
  filter(era == "curr", year == 100) %>%
  mutate(
    pop = case_when(
      lifestage == "spawners" ~ "steelhead",
      lifestage == "spawners_winter" ~ "wsteelhead",
      lifestage == "spawners_summer" ~ "ssteelhead"
    )) %>%
  filter(pop != "steelhead", scenario != "Delta") %>% # take out the totals
  group_by(pop) %>%
  mutate(
    n_cur = n[scenario == 'Current'],
    diff_perc = (n - n_cur)/n_cur,
    diff_str = scales::percent(
      na_if(diff_perc, 0),
      accuracy = 1, prefix = "+"),
    scenario = factor(
      scenario,
      levels = scenarios$scenario)) %>%
  make_diag_barplot()

ggsave(sprintf("%s/barplot_STL_steelhead_%s.png", out_path, commit_date),
       width = 6, height = 8)
